# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#==========================================================================


import time
import logging
from .naeusb import packuint32

class FPGA(object):

    CMD_FPGA_STATUS = 0x15
    CMD_FPGA_PROGRAM = 0x16

    def __init__(self, usb, timeout=200):
        self.sendCtrl = usb.sendCtrl
        self.readCtrl = usb.readCtrl
        self._usb = usb
        self._timeout = timeout

    def isFPGAProgrammed(self):
        """
        Return True/False about FPGA status
        """

        status = self.readCtrl(self.CMD_FPGA_STATUS, dlen=4)

        if status[0] & 0x01:
            return True
        else:
            return False

    def FPGAProgram(self, bitstream=None, exceptOnDoneFailure=True):
        """
        Program FPGA with a bitstream, or if not bitstream passed just erases FPGA
        """

        # Erase the FPGA by toggling PROGRAM pin, setup
        # NAEUSB chip for FPGA programming
        self.sendCtrl(self.CMD_FPGA_PROGRAM, 0xA0)
        time.sleep(0.001)
        self.sendCtrl(self.CMD_FPGA_PROGRAM, 0xA1)

        time.sleep(0.001)

        # Download actual bitstream now if present
        if bitstream:
            # Run the download which should program FPGA
            self._FPGADownloadBitstream(bitstream)

            wait = 5
            while wait > 0:
                # Check the status a few times
                programStatus = self.isFPGAProgrammed()
                if programStatus:
                    break
                time.sleep(0.001)
                wait -= 1

            # Exit FPGA programming mode
            self.sendCtrl(self.CMD_FPGA_PROGRAM, 0xA2)

            if programStatus == False and exceptOnDoneFailure:
                raise IOError("FPGA Done pin failed to go high, bad bitstream?", bitstream)

            return programStatus
        else:
            # No bitstream, exit programming mode
            self.sendCtrl(self.CMD_FPGA_PROGRAM, 0xA2)
            return False

    def _FPGADownloadBitstream(self, fwFileLike):
        """
        Performs actual bitstream download, do not call directly, call FPGAProgram
        """

        transactionBytes = 2048
        t0 = 0

        buffer_ = [None] * int(16 * 1024 * 1024 / transactionBytes)
        size = 0

        # Read entire thing in
        with fwFileLike as f:
            inputStream = f.read()
        streamCnt = 0

        # Might need a few extra CCLKs at end to finish off, and as written elsewhere this is done with DO=1
        # Perhaps micro should add these instead? For now this should be reliable enough (things worked even w/o this it seemed, so this is
        # a just in case item)
        inputStream += bytes([0xff] * 32)

        inputStream = inputStream[0x7C:]

        j = transactionBytes
        for i in range(0, len(buffer_)):
            if j != transactionBytes: break
            buffer_[i] = bytearray(inputStream[streamCnt:(streamCnt + transactionBytes)])
            streamCnt += transactionBytes

            j = len(buffer_[i])

            if j < transactionBytes and j % 64 == 0:
                j += 1
            size += j

            # if j < transactionBytes and j % 64 == 0:
            #    j += 1
            # size += j

            # print i,
            # print " ",
            # print len(buffer_)

        if size < 64 or size % 64 == 0:
            raise ValueError("Invalid file size: " + str(size))

        tries = 1

        while tries > 0:
            # self.resetFpga()
            try:
                # t0 = -Date().getTime()
                bs = 0
                cs = 0

                for i in range(0, len(buffer_)):
                    if i * transactionBytes >= size:
                        break
                    j = size - i * transactionBytes
                    if j > transactionBytes:
                        j = transactionBytes

                    self._usb.writeBulkEP(buffer_[i])

                    bs += j
                    for k in range(0, len(buffer_[i])):
                        cs = (cs + (buffer_[i][k] & 0xff)) & 0xff

                # self.getFpgaState()
                # if not self.fpgaConfigured:
                #    raise IOError("FPGA configuration failed: DONE pin does not go high (size=" + self.fpgaBytes + " ,  " + (bs - self.fpgaBytes) + " bytes got lost;  checksum=" + self.fpgaChecksum + " , should be " + cs + ";  INIT_B_HIST=" + self.fpgaInitB + ")")

                # if self.enableExtraFpgaConfigurationChecks:
                #    if self.fpgaBytes != 0 and self.fpgaBytes != bs:
                #        System.err.println("Warning: Possible FPGA configuration data loss: " + (bs - self.fpgaBytes) + " bytes got lost")
                #    if self.fpgaInitB != 222:
                #        System.err.println("Warning: Possible Bitstream CRC error: INIT_B_HIST=" + self.fpgaInitB)
                tries = 0
                # t0 += Date().getTime()
            except IOError as e:
                if tries > 1:
                    print(("Warning: " + str(e) + ": Retrying it ..."))
                else:
                    raise
            tries -= 1
        # time.sleep(0.1)
        return t0

        # def detectBitstreamBitOrder(self, buf):
        #    """ Determine what bit-order bitstream is in by looking for magic bytes """
        #
        #    i = 0
        #    while i < len(buf):
        #        if ((buf[i] & 255) == 0xaa) and ((buf[i + 1] & 255) == 0x99) and ((buf[i + 2] & 255) == 0x55) and ((buf[i + 3] & 255) == 0x66):
        #            return 1
        #        if ((buf[i] & 255) == 0x55) and ((buf[i + 1] & 255) == 0x99) and ((buf[i + 2] & 255) == 0xaa) and ((buf[i + 3] & 255) == 0x66):
        #            return 0
        #        i += 1
        #    print("Warning: Unable to determine bitstream bit order: no signature found")
        #    return 0



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from . import _OpenADCInterface as openadc
from chipwhisperer.common.utils import util, timer


class OpenADCQt(object):
    _name= 'OpenADC'

    def __init__(self):
        self.dataUpdated = util.Signal()


        self.offset = 0.5
        self.ser = None
        self.sc = None
        self.parm_hwinfo = None
        self.parm_gain = None
        self.parm_trigger = None
        self.parm_clock = None

        self.datapoints = []

        self.timerStatusRefresh = timer.Timer()
        self.timerStatusRefresh.timeout.connect(self.statusRefresh)

    def setEnabled(self, enabled):
        pass

    def statusRefresh(self):
        pass

    def processData(self, data):
        fpData = []

        if data[0] != 0xAC:
            logging.warning("Unexpected sync byte: 0x%x" % data[0])
            return None

        for i in range(2, len(data)-1, 2):
            if (0x80 & data[i + 1]) or ((0x80 & data[i + 0]) == 0):
                logging.error('Error at byte ' + str(i) + '. Bytes: %x %x' % (data[i], data[i+1]))
                return None

            #Convert
            intpt = data[i + 1] | ((data[i+0] & 0x07) << 7)

            #input validation test: uncomment following and use
            #ramp input on FPGA
            ##if (intpt != lastpt + 1) and (lastpt != 0x3ff):
            ##    print "intpt: %x lstpt %x\n"%(intpt, lastpt)
            ##lastpt = intpt;

            fpData.append(float(intpt) / 1024.0 - self.offset)

        return fpData

    def arm(self):
        self.sc.arm()

    def startCaptureThread(self):
        self.sc.startCaptureThread()

    def read(self, numberPoints=None, channelNr=0):
        if numberPoints == None:
            numberPoints = self.parm_trigger.samples

        try:
            self.datapoints = self.sc.readData(numberPoints)
            if self.datapoints is None or len(self.datapoints) == 0:
                return True #effectively a timeout for now
        except IndexError as e:
            raise IOError("Error reading data: %s" % str(e))

        self.dataUpdated.emit(channelNr, self.datapoints, -self.parm_trigger._get_presamples(True), self.parm_clock._adcSampleRate())
        return False


    def trigger_duration(self):
        return self.parm_trigger.duration()

    def capture(self, offset=None):
        timeout = self.sc.capture(offset)
        timeout2 = self.read()

        return timeout or timeout2

    def reset(self):
        self.sc.setReset(True)

    def test(self):
        self.sc.testAndTime()

    def con(self, ser):
        self.ser = ser
        # See if device seems to be attached
        self.sc = openadc.OpenADCInterface(self.ser)

        self.parm_hwinfo = openadc.HWInformation(self.sc)

        self.parm_gain = openadc.GainSettings(self.sc)

        self.parm_trigger = openadc.TriggerSettings(self.sc)

        self.parm_clock = openadc.ClockSettings(self.sc, hwinfo=self.parm_hwinfo)

        deviceFound = False
        numTries = 0

        # Try a few times
        while(deviceFound == False):
            if self.sc.devicePresent():
                deviceFound = True
                break

            numTries += 1
            if numTries == 5:
                try:
                    portname = self.ser.name
                except:
                    portname = "UNKNOWN"
                self.close()

                raise IOError("Opened port %s but failed to find OpenADC" % portname)

        self.setEnabled(True)

    def close(self):
        self.ser = None
        self.parm_hwinfo = None

        self.parm_gain = None

        self.parm_trigger = None

        self.parm_clock = None
        self.sc = None

    def __del__(self):
        self.close()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np

from ._base import PreprocessingBase
from collections import OrderedDict


class ResyncSAD(PreprocessingBase):
    """Resynchronize traces by shifting them to match in a certain window.

    This method of pre-processing uses the Sum of Absolute Difference
    (SAD) algorithm, also known as the Sum of Absolute Error. Uses a portion
    of one of the traces as a reference. This reference is then slid over the
    input "window" of each trace, and the amount of the shift resulting in the
    minimum SAD criteria is used as the shift amount for each trace.

    Args:
        trace_source (Project): A project containing the traces to preprocess.
    """
    _name = "Resync: Sum-of-Difference"
    _description = "Minimizes the 'Sum of Absolute Difference' (SAD), also known as 'Sum of Absolute Error'. Uses "\
                  "a portion of one of the traces as the 'reference'. This reference is then slid over the 'input "\
                  "window' for each trace, and the amount of shift resulting in the minimum SAD criteria is selected "\
                  "as the shift amount for that trace."

    def __init__(self, trace_source=None):
        PreprocessingBase.__init__(self, trace_source, name=None)
        self._rtrace = None
        self._debugReturnSad = False
        self._wdStart = 0
        self._wdEnd = 1
        self._maxshift = 1
        self._init_not_done = True

    def _setRefTrace(self, num):
        self._rtrace = num
        self._calculateRef()

    def _getRefTrace(self):
        return self._rtrace

    def _setMaxShift(self, shift):
        self._maxshift = shift

    def _getMaxShift(self):
        return self._maxshift

    @property
    def ref_trace(self):
        """The trace being used as a reference.

        Raises:
            TypeError: If attempting to set to non integer value.
        """
        return self._getRefTrace()

    @ref_trace.setter
    def ref_trace(self, num):
        if not isinstance(num, int):
            raise TypeError("Expected int; got %s" % type(num), num)
        self._setRefTrace(num)

    @property
    def max_shift(self):
        """Maximum amount to shift around a trace around the target_window"""

        return self._getMaxShift()

    @max_shift.setter
    def max_shift(self, m):
        if not isinstance(m, int):
            raise TypeError("Expected int; got %s" % type(m), m)
        self._setMaxShift(m)

    def _setWindow(self, window):
        self._wdStart, self._wdEnd = window
        self._calculateRef()

    def _getWindow(self):
        return (self._wdStart, self._wdEnd)

    @property
    def target_window(self):
        """Section of the trace we are trying to minimize SAD for.

        This must be a tuple of (first point, last point).

        Raises:
            TypeError: If attempting to set to a non tuple or if points are not
                integers.
        """
        return self._getWindow()

    @target_window.setter
    def target_window(self, win):
        if not isinstance(win, tuple):
            raise TypeError("Expected tuple; got %s" % type(win), win)
        if not isinstance(win[0], int):
            raise TypeError("Expected int; got %s" % type(win[0]), win[0])
        if not isinstance(win[1], int):
            raise TypeError("Expected int; got %s" % type(win[1]), win[1])
        self._setWindow(win)

    @property
    def input_window(self):
        raise NotImplementedError("SAD interface changed. See target_window now.")

    @input_window.setter
    def input_window(self, _):
        raise NotImplementedError("SAD interface changed. See target_window now.")

    def updateLimits(self):
        if self._traceSource:
            self.findParam('windowpt').setLimits((0, self._traceSource.numPoints()-1))
            self._calculateRef()

    def setOutputSad(self, enabled):
        self._debugReturnSad = enabled

    def get_trace(self, n):
        if self.enabled:
            if self._init_not_done:
                self._calculateRef()
                self._init_not_done = False

            trace = self._traceSource.get_trace(n)
            if trace is None:
                return None
            sad = self._findSAD(trace)

            if self._debugReturnSad:
                return sad

            if len(sad) == 0:
                return None

            newmaxloc = np.argmin(sad)
            maxval = min(sad)
            #if (maxval > self.refmaxsize * 1.01) | (maxval < self.refmaxsize * 0.99):
            #    return None

            if maxval > self.maxthreshold:
                return None

            diff = newmaxloc-self.refmaxloc
            if diff < 0:
                trace = np.append(np.zeros(-diff), trace[:diff])
            elif diff > 0:
                trace = np.append(trace[diff:], np.zeros(diff))
            return trace
        else:
            print("Not enabled")
            return self._traceSource.get_trace(n)

    getTrace = get_trace

    def _calculateRef(self):
        try:
            self.calcRefTrace(self._rtrace)
        #Probably shouldn't do this, but deals with user enabling preprocessing
        #before trace management setup
        except ValueError:
            pass

    def _findSAD(self, inputtrace):
        sadlen = self._maxshift * 2
        sadarray = np.ones(sadlen)*1E6

        wdlen = self._wdEnd - self._wdStart

        #Shift values
        minshift = -self._maxshift
        maxshift = self._maxshift

        if minshift + self._wdStart < 0:
            raise ValueError("Invalid size or maximum shift, starting search location is < 0")

        if maxshift + self._wdEnd > len(inputtrace):
            raise ValueError("Invalid size or maximum shift, ending search location is outside trace")

        for ptoffset in range(minshift, maxshift):
            ptstart = ptoffset + self._wdStart
            diff_data = inputtrace[ptstart:(ptstart+wdlen)] - self.reftrace
            abs_data = np.abs(diff_data)
            sadarray[ptoffset + self._maxshift] = np.sum(abs_data)
        return sadarray

    def calcRefTrace(self, tnum):
        if self.enabled == False:
            return

        self.reftrace = self._traceSource.get_trace(tnum)[self._wdStart:self._wdEnd]
        sad = self._findSAD(self._traceSource.get_trace(tnum))
        self.refmaxloc = np.argmin(sad)
        self.refmaxsize = min(sad)
        self.maxthreshold = np.mean(sad)


    def _dict_repr(self):
        dict = OrderedDict()
        dict['enabled'] = self.enabled
        dict['ref_trace'] = self.ref_trace
        dict['max_shift'] = self.max_shift
        dict['target_window']    = self.target_window
        return dict



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2018, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
from ._base import PreprocessingBase
from matplotlib.mlab import find
import scipy.signal as sig
from chipwhisperer.common.utils.parameter import setupSetParam


class ResyncResampleZC(PreprocessingBase):
    """
    Resync using Resampling based on Zero-Crossing Bins.
    """
    _name = "Resync: Resample based on Zero-Crossing"
    _description = "Deal with resampling 'bins' based on zero-crossing detection."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._rtrace = 0
        self._zcoffset = 0.0
        self._binlen = 0

        self.params.addChildren([
            {'name':'Ref Trace', 'key':'reftrace', 'type':'int', 'get':self._getRefTrace, 'set':self._setRefTrace},
            {'name':'Zero-Crossing Level', 'key':'zclevel', 'type':'float', 'get':self._getZcLevel, 'set':self._setZcLevel},
            {'name':'Bin Sample Length', 'key':'binlen', 'type':'int', 'limits':(0, 10000), 'get':self._getBinLength, 'set':self._setBinLength},
        ])

    @setupSetParam("Ref Trace")
    def _setRefTrace(self, num):
        self._rtrace = num
        self._calculateRef()

    def _getRefTrace(self):
        return self._rtrace

    @property
    def ref_trace(self):
        """The trace being used as a reference.

        Setter raises TypeError unless value is an integer."""
        return self._getRefTrace()

    @ref_trace.setter
    def ref_trace(self, num):
        if not isinstance(num, int):
            raise TypeError("Expected int; got %s" % type(num), num)
        self._setRefTrace(num)

    @setupSetParam("Zero-Crossing Level")
    def _setZcLevel(self, offset):
        self._zcoffset = offset
        self._calculateRef()

    def _getZcLevel(self):
        return self._zcoffset

    @property
    def zc_offset(self):
        """The level to use as the zero-crossing point.

        Setter raises TypeError if value isn't a float.
        """
        return self._getZcLevel()

    @zc_offset.setter
    def zc_offset(self, offset):
        if not isinstance(offset, float):
            raise TypeError("Expected float; got %s" % type(offset), offset)
        self._setZcLevel(offset)

    @setupSetParam("Bin Sample Length")
    def _setBinLength(self, offset):
        self._zcoffset = offset
        self._calculateRef()

    def _getBinLength(self):
        return self._zcoffset

    @property
    def bin_length(self):
        """The number of samples to be resampled at once. This should be close
        to the number of samples between two zero-crossings.

        If 0, a bin length is automatically calculated.

        Setter raises TypeError if value isn't an int.
        """
        return self._getBinLength()

    @bin_length.setter
    def bin_length(self, len):
        if not isinstance(len, int):
            raise TypeError("Expected float; got %s" % type(len), len)
        self._setBinLength(len)

    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None
            
            trace = trace - self._zcoffset
    
            ind = self._findZerocrossing(trace)
            return self._resampleResize(trace, ind, self._binlen)
        else:
            return self._traceSource.getTrace(n)
   
    def _calculateRef(self):
        try:
            self.calcRefTrace(self._rtrace)
        #Probably shouldn't do this, but deals with user enabling preprocessing
        #before trace management setup
        except ValueError:
            pass

    def calcRefTrace(self, tnum):
        #If not enabled stop
        if self.enabled == False:
            return
        
        if self._binlen == 0:
            self._reftrace = self._traceSource.getTrace(tnum) - self._zcoffset
            ind = self._findZerocrossing(self._reftrace)
            self._binlen = self._findAvgLength(ind)

    def _findZerocrossing(self, a):
        indices = find((a[1:] >= 0) & (a[:-1] < 0))
        return indices

    def _findAvgLength(self, indices):
        diff = 0
        num = 0

        for i in range(1, len(indices)):
            diff += indices[i] - indices[i - 1]
            num += 1

        return diff / num

    def _resampleResize(self, data, indices, targlen):
        targdata = np.zeros(targlen * len(indices))

        # Shift each segment to fit this size, let it ride
        for i in range(1, len(indices)):
            targdata[(i - 1) * targlen:i * targlen] = sig.resample(data[indices[i - 1]:indices[i]], targlen)

        return targdata



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from chipwhisperer.common.utils.parameter import Parameterized

class ProtocolTemplate(Parameterized):
    _name= 'Smartcard Protocol'

    def __init__(self):
        self.hw = None
        self.getParams()
        # self.getParams().addChildren(
        #     {'name':'Example Parameter', 'type':'int', 'value':5, 'set':self.setSomething}
        # )

    def setSomething(self):
        pass

    def setReaderHardware(self, hw):
        self.hw = hw

    def init(self):
        pass

    def reinit(self):
        pass

    def loadEncryptionKey(self, key):
        pass

    def loadInput(self, inputtext):
        self.input = inputtext

    def isDone(self):
        return True

    def readOutput(self):
        return ""

    def go(self):
        pass



"""Trace Preprocessing Classes

Classes:
* AddNoiseJitter
* AddNoiseRandom
* CacheTraces
* DecimationClockRecovery - Not working
* DecimationFixed
* DigitalFilter
* PassThrough
* ResyncCrossCorrelation
* ResyncDTW
* ResyncPeakDetect
* ResyncResampleZC
* ResyncSAD
"""

from .add_noise_jitter import AddNoiseJitter
from .add_noise_random import AddNoiseRandom
#from .decimation_clock_recovery import DecimationClockRecovery broken without matplotlib/cycler
from .decimation_fixed import DecimationFixed
from .digital_filter import Filter as DigitalFilter
from .pass_through import PassThrough
from .resync_cross_correlation import ResyncCrossCorrelation
from .resync_dtw import ResyncDTW
from .resync_peak_detect import ResyncPeakDetect
#from .resync_resample_zc import ResyncResampleZC broken without matplotlib/cycler
from .resync_sad import ResyncSAD



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np

from ..algorithmsbase import AlgorithmsBase


class AttackCPA_Bayesian(AlgorithmsBase):
    """
    Bayesian CPA. NOT WORKING!!
    """
    _name = "Bayesian CPA"

    def __init__(self):
        AlgorithmsBase.__init__(self)
        self.updateScript()

    def addTraces(self, traceSource, tracerange, progressBar=None, pointRange=None, algo="log", tracesLoop=None):
        keyround=self.keyround
        modeltype=self.modeltype
        brange=self.brange

        # foundkey = []

        self.all_diffs = list(range(0,16))

        pbcnt = 0
        if progressBar:
            progressBar.setStatusMask("Byte %02d: Hyp=%02x")
            progressBar.setMaximum(len(brange) * 256)

        # Load all traces
        data = []
        textins = []
        textouts = []
        knownkeys = []
        for i in range(tracerange[0], tracerange[1]):
            # Handle Offset
            tnum = i + tracerange[0]
            try:
                data.append(traceSource.getTrace(tnum))
                textins.append(traceSource.getTextin(tnum))
                textouts.append(traceSource.getTextout(tnum))
                knownkeys.append(traceSource.getKnownKey(tnum))
            except Exception as e:
                progressBar.abort(e.message)
                return

        traces = np.array(data)
        textins = np.array(textins)
        textouts = np.array(textouts)

        #For all bytes of key
        for bnum in brange:
            diffs = [0]*256

            #For each 0..0xFF possible value of the key byte
            for key in range(0, 256):
                #Initialize arrays & variables to zero
                sumstd = np.zeros(len(traces[0,:]), dtype=np.float64)
                hyp = [0] * len(traces[:, 0])

                #Generate hypotheticals
                for tnum in range(len(traces[:,0])):

                    if len(textins) > 0:
                        pt = textins[tnum]

                    if len(textouts) > 0:
                        ct = textouts[tnum]

                    if keyround == "first":
                        ct = None
                    elif keyround == "last":
                        pt = None
                    else:
                        raise ValueError("keyround invalid")

                    #Generate the output of the SBOX
                    if modeltype == "Hamming Weight":
                        hypint = self.model.HypHW(pt, ct, key, bnum)
                    elif modeltype == "Hamming Distance":
                        hypint = self.model.HypHD(pt, ct, key, bnum)
                    else:
                        raise ValueError("modeltype invalid")

                    hyp[tnum] = hypint

                hyp = np.array(hyp)

                #Mean & STD-DEV of hypothesis
                meanh = np.mean(hyp, dtype=np.float64)
                stddevh = np.std(hyp, dtype=np.float64)

                #Mean & STD-DEV of all points in trace
                meant = np.mean(traces, axis=0, dtype=np.float64)
                stddevt = np.std(traces, axis=0, dtype=np.float64)

                #For each trace, do the following
                for tnum in range(len(traces[:,0])):
                    #Make both zero-mean
                    hdiff = np.float64(hyp[tnum]) - meanh
                    tdiff = np.array((np.array(traces[tnum,:], dtype=np.float64) - meant), dtype=np.float64)

                    #Scale by std-dev
                    hdiff = hdiff / np.float64(stddevh)
                    tdiff = tdiff / np.array(stddevt, dtype=np.float64)

                    # Calculate error term
                    error = (hdiff - tdiff)
                    sumstd = sumstd + pow(error, 2)

                q = len(traces[:,0])

                if algo == "original":
                    #Original Algorithm
                    sumstd = pow(np.sqrt((sumstd/q)), -q)
                elif algo == "log":
                    #LOG Algorithm
                    sumstd = -q * ( (0.5*np.log(sumstd)) - np.log(q))
                else:
                    raise RuntimeError("algo not defined")

                diffs[key] = sumstd

                if progressBar:
                    progressBar.updateStatus(pbcnt, (bnum, key))
                    if progressBar and progressBar.wasAborted():
                        return
                pbcnt = pbcnt + 1

            #Gotten all stddevs - now process algorithm

            #Original algorithm
            if algo == "original":
                sums = np.sum(diffs, axis=0, dtype=np.float64)
                for key in range(0, 256):
                    diffs[key] = diffs[key] / sums

            elif algo == "log":
            #Logarithm Algorithm
                summation = np.zeros(len(traces[0,:]), dtype=np.float64)
                for key in range(1, 256):
                    diff = diffs[key] - diffs[0]
                    summation = summation + np.exp(diff)
                summation = summation + 1
                summation = np.log(summation)
                summation = summation + diffs[0]

                for key in range(0, 256):
                    diffs[key] = diffs[key] - summation

            else:
                raise RuntimeError("algo not defined")

            self.all_diffs[bnum] = diffs
        self.algo = algo
            #From all the key candidates, select the largest difference as most likely
            #foundbyte = diffs.index(max(diffs))
            #foundkey.append(foundbyte)
#            print "%2x "%foundbyte,

    def _getResult(self, bnum, hyprange=None):
        if hyprange == None:
            hyprange = list(range(0,256))
        return [self.all_diffs[bnum][i] for i in hyprange]

    def getDiff(self, bnum, hyprange=None):
        if self.algo == "original":
            return self._getResult(bnum, hyprange)
        elif self.algo == "log":
            return np.exp(self._getResult(bnum, hyprange))
        else:
            raise RuntimeError("algo not defined")

    def getStatistics(self):
        t = [0]*16
        for i in self.brange:
            t[i] = self.getDiff(i)
        return t



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import sys
from chipwhisperer.analyzer.attacks.cpa_algorithms.progressive import CPAProgressive
from ._base import AttackBaseClass


class CPA(AttackBaseClass):
    """Correlation Power Analysis Attack

    Before running attack, set a leak model (set_leak_model()). By default
    attacks all keys using all traces.
    Basic usage:

    .. code:: python
        >>> import chipwhisperer as cw
        >>> import chipwhisperer.analyzer as cwa
        >>> project = cw.open_project("project.cwp")
        >>> attack = cwa.cpa(project.trace_manager())
        >>> leak_model = cwa.AES128(cw.aes128leakage.SBox_output)
        >>> attack.set_leak_model(leak_model)
        >>> attack.process_traces()
    """
    _name = "CPA"

    def __init__(self):
        self._analysisAlgorithm = CPAProgressive()
        AttackBaseClass.__init__(self)
        self.updateScript()

    def updateScript(self, _=None):
        AttackBaseClass.updateScript(self)

        analysAlgoStr = sys.modules[self._analysisAlgorithm.__class__.__module__].__name__ + '.' + self._analysisAlgorithm.__class__.__name__
        model_path = None
        cryptoalg = None
        hwmodel = None


    def processKnownKey(self, inpkey):
        if inpkey is None:
            return None

        if hasattr(self.attack, 'processKnownKey'):
            return self.attack.processKnownKey(inpkey)
        else:
            return inpkey

    def getStatistics(self):
        return self.attack.getStatistics()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
import logging

__author__ = "Colin O'Flynn"

import sys
import os
#import pstats, cProfile #For profiling support (not 100% needed)

try:
    from configobj import ConfigObj  # import the module
except ImportError:
    print("ERROR: configobj (https://pypi.python.org/pypi/configobj/) is required for this program")
    sys.exit()


def makeAttrDict(sectionname, modulename, paramSpecs):
    """
    Converts a parameter tree configuration setup into an attribute dictionary. Requires specific setup
    of parameter tree, assumes passed a single-element list, with first element having 'children'. No
    sub-groups allowed. Any element without a 'value' key is skipped.
    """
    
    ps = paramSpecs[0]['children']
    
    attrDict = {
                "sectionName":sectionname,
                "moduleName":modulename,
                "module":None,
                "values":{},
                }    
    order = 0    
    for p in ps:
        try:
            edit = not p["readonly"]
        except KeyError:
            edit = True
            
        try:
            attrDict["values"][p["key"]] = {"order":order, "value":p["value"], "desc":p["name"], "changed":False, "editable":edit}
        except KeyError:
            #Silently skip anything without a 'value' attribute given, which forms the initial value
            pass
        
        order += 1
        
    return attrDict
    
   
class TraceContainerConfig(object):
    """
    This class holds configuration data for a TraceContainer class. This is split into a separate class
    partially for historical reasons, since the old code had this as a separate class.
    
    This class is responsible for reading/writing the .cfg file associated with any traces. Even if we are
    using non-native trace format, the traces will STILL have a ChipWhisperer-specific .cfg file written
    with them.
    
    This choice means you aren't forced to import traces into ChipWhisperer-only format, and can for example
    you may wish to keep them in some MATLAB workspace format instead.
    """   
        
    ## Notes on dictionary:
    # order = order of display, needed since dictionary is unordered
    # value = value of variable
    # desc = description of variable
    # changed = has value changed since being written to disk?
    # headerLabel = short description/name used for header label in trace management dialog
    #

    def __init__(self, configfile=None):
        """If a config file is given, will attempt to load that file"""
        self.attrDictInst = {
                "sectionName":"Trace Config",
                "moduleName":"native",
                "module":None,
                "values":{
                    "format":{"order":0, "value":"native", "desc":"Native Format Type", "changed":False, "headerLabel":"Format", "editable":False},
                    "numTraces":{"order":1, "value":0, "desc":"Number of Traces in File", "changed":False, "headerLabel":"# Traces", "editable":False},
                    "numPoints":{"order":2, "value":0, "desc":"Number of Points per trace, assuming uniform", "changed":False, "headerLabel":"# Points", "editable":False},
                    "date":{"order":3, "value":"1997-01-28 17:05:00", "desc":"Date of Capture YYYY-MM-DD HH:MM:SS Format", "changed":False, "headerLabel":"Date and Time of Capture", "editable":False},
                    "prefix":{"order":4, "value":None, "desc":"Prefix of all files if applicable", "changed":False},
                    "targetHW":{"order":5, "value":"unknown", "desc":"Description of Target (DUT) Hardware", "changed":False, "headerLabel":"Target Hardware", "editable":True},
                    "targetSW":{"order":6, "value":"unknown", "desc":"Description of Target (DUT) Software", "changed":False, "headerLabel":"Target Software", "editable":True},
                    "scopeName":{"order":7, "value":"unknown", "desc":"Scope Model/Description", "changed":False, "headerLabel":"Scope Module Name", "editable":True},
                    "scopeSampleRate":{"order":8, "value":0, "desc":"Sample Rate (s/sec)", "changed":False, "headerLabel":"Sample Rate", "editable":True},
                    "scopeYUnits":{"order":9, "value":0, "desc":"Units of Y Points", "changed":False, "editable":True},
                    "scopeXUnits":{"order":10, "value":0, "desc":"Units of X Points", "changed":False, "editable":True},
                    "notes":{"order":11, "value":"", "desc":"Additional Notes about Capture Setup", "changed":False, "headerLabel":"Notes", "editable":True}                
                    },
                }
    
        self.attrList = [self.attrDictInst]
        self.mappedRange = None
        self.trace = None
        if configfile is not None:
            configfile = os.path.normpath(configfile)
        self._configfile = configfile

        #Attempt load/sync
        self.loadTrace(configfile)
        
    def module(self, attr, moduleName=None):
        """Given an attribute & possibly module name, return reference to module dictionary"""
        module = None
                
        if moduleName == None:
            for i in self.attrList:
                if attr in i["values"]:
                    module = i
                    break
        else:
            for i in self.attrList:
                if (i["moduleName"] == moduleName) | (i["sectionName"] == moduleName):
                    if attr in i["values"]:
                        module = i
                        break
                
        if module == None:
            raise ValueError("Invalid attribute: %s"%attr)
        
        return module
        
    def attrHeaderValues(self):
        """Used to list all values in {"name":name, "header":header title, "desc":description} dictionary"""
        lst = []
        
        for i in self.attrList:
            for p in i["values"].keys():
                try:
                    lst += [{ "name":p, "header":i["values"][p]["headerLabel"], "desc":i["values"][p]["desc"], "order":i["values"][p]["order"] }]
                except KeyError:
                    #If we don't have headerLabel type, we skip this item
                    pass
        
        lst = sorted(lst, key=lambda k: k["order"]) 
        
        return lst
     
    def setConfigFilename(self, fname):
        """Set the config filename, WITHOUT syncronizing internal DB to File. Use load or save for this feature."""
        if fname is not None:
            fname = os.path.normpath(fname)
        self._configfile = fname
        
    def configFilename(self):
        """Get Config Filename"""
        return self._configfile
     
    def attr(self, attr, moduleName=None):
        """Get value of attribute specified from internal DB"""       
        return self.attrDict(attr, moduleName)["value"]
    
    def attrDict(self, attr, moduleName=None):
        """Get dictionary of attribute specified, includes additional info such as flags"""
        mod = self.module(attr,moduleName)        
        return mod["values"][attr]
        
    def setAttr(self, attr, value, moduleName=None):
        """Set value of attribute in internal DB"""
        mod = self.module(attr,moduleName)
        mod["values"][attr]["value"] = value
        mod["values"][attr]["changed"] = True 
               
    def syncFile(self, sectionname=None):
        """
        Sync the internal DB cache to the .cfg file specified already, basically this can be a save or load.
        
        The CHANGED flag is used to decide what to do. If the 'changed' flag is FALSE in the internal DB,
        the disk has priority. Any attributes not in the disk .cfg file will be taken from the internal DB.
        
        If the 'changed' flag of an attribute is TRUE, the internal DB will overwrite the disk file.
        """

        for ad in self.attrList:

            # If user only wishes to update a certain section do that
            if sectionname is not None:
                if ad["sectionName"] != sectionname:
                    continue

            sn = ad["sectionName"]
            
            #Does section exist?
            try:
                self.config[sn]
                logging.debug('Section %s found' % sn)
            except KeyError:
                self.config[sn] = {}
                logging.debug('Section %s not found' % sn)
                
            #Check each item
            for item in ad["values"].keys():
                try:
                    self.config[sn][item]
                    
                    #Check priority
                    if ad["values"][item]["changed"] == False:                        
                        ad["values"][item]["value"] = self.config[sn][item]
                        logging.debug('%s from configfile = %s' % (item, self.config[sn][item]))
                    else:
                        self.config[sn][item] = ad["values"][item]["value"]
                        logging.debug('%s from cache = %s' % (item, ad["values"][item]["value"]))
                
                except KeyError:
                    self.config[sn][item] = ad["values"][item]["value"]
                    logging.debug('%s missing in cfg file' % item)
        
    def loadTrace(self, configfile=None):
        """Load config file. Syncs internal DB to File"""
        if configfile:
            self._configfile = os.path.normpath(configfile)
            if not os.path.isfile(configfile):
                raise Exception("Error: Trace set config file doesn't exists: " + configfile)

        self.config = ConfigObj(self._configfile)
        self.syncFile()
        
    def saveTrace(self, configfile = None):
        """Save internal DB to Config File"""
        self.config.filename = self._configfile
        self.syncFile()     
        self.config.write()

    def checkTraceItem(self, itemname, localitem, changedlist):
        if localitem != self.__dict__[itemname]:                      
            changedlist.append(itemname)
            return True
        return False        

    def checkTrace(self, configfile = None):
        if configfile == None:
            configfile = self.configfile
      
        changed = True
        
        #TODO: THIS

        return changed



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from chipwhisperer.common.utils import util
from chipwhisperer.common.utils.tracesource import TraceSource


class ScopeTemplate(object):
    _name = "None"

    def __init__(self):
        self.connectStatus = False

    def _getNAEUSB(self):
        raise Warning("Can't find low level USB interface for scope " + self.getName())

    def dcmTimeout(self):
        pass

    def setAutorefreshDCM(self, enabled):
        pass

    def setCurrentScope(self, scope):
        pass

    def newDataReceived(self, channelNum, data=None, offset=0, sampleRate=0):
        pass

    def getStatus(self):
        return self.connectStatus

    def con(self, sn=None):
        if self._con(sn):
            self.connectStatus = True

    def _con(self, sn=None):
        raise NotImplementedError("Scope \"" + self.getName() + "\" does not implement method " + self.__class__.__name__ + ".con()")

    def dis(self):
        if self._dis():
            pass
        self.connectStatus = False

    def _dis(self):
        raise NotImplementedError("Scope \"" + self.getName() + "\" does not implement method " + self.__class__.__name__ + ".dis()")

    def arm(self):
        """Prepare the scope for capturing"""
        # NOTE - if reimplementing this, should always check for connection first
        # if self.connectStatus.value() is False:
        #     raise Exception("Scope \"" + self.getName() + "\" is not connected. Connect it first...")
        # raise NotImplementedError("Scope \"" + self.getName() + "\" does not implement method " + self.__class__.__name__ + ".arm()")
        pass

    def capture(self):
        """Capture one trace and returns True if timeout has happened."""

        # NOTE: If you have a waiting loop (waiting for arm), call the function util.updateUI() inside that loop to keep
        #       the UI responsive:
        #
        # while self.done() == False:
        #     time.sleep(0.05)
        #     util.updateUI()
        pass

    def get_name(self):
        return self._name

    getName = util.camel_case_deprecated(get_name)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2018, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.github.com/newaetech/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np
from chipwhisperer.common.api.ProjectFormat import Project
from chipwhisperer.common.traces import Trace


def calculate_snr(input, leak_model, bnum=0, db=True):
    """Calculate the SNR based on the leakage model.

    Uses same leakage model as the CPA attack.

    Args:
        input (Iterable of :class:`Traces <chipwhisperer.common.traces.Trace>`): An iterable of traces.
        leak_model (ModelsBase): A leakage model selected from
            :data:`leakage_models <chipwhisperer.analyzer.leakage_models>`.
        bnum (int): Byte number used for leakage model.
        bd (bool): Return signal-to-noise ratio in decibals.
    """

    textin = None
    textout = None
    key = None
    trace = None

    hwarray = []

    tm = None

    ntrace = len(input)
    npoints = len(input[0].wave)

    for tnum in range(0, ntrace):
        trace = input[tnum].wave
        textin = input[tnum].textin
        textout = input[tnum].textout
        key = input[tnum].key

        state = {'knownkey':key}

        leakage = leak_model.leakage(textin, textout, None, bnum, state)

        while leakage >= len(hwarray):
            hwarray.append([])

        hwarray[leakage].append(trace)

    hwmean = np.zeros((len(hwarray), npoints))

    for i in range(0, len(hwarray)):
        hwmean[i] = np.mean(hwarray[i], axis=0)

    inc_list = []
    best_choice = -1
    best_choice_len = 0
    for i in range(0, len(hwarray)):
        num_elements = len(hwarray[i])
        if num_elements > 0:
            inc_list.append(i)
            #Figure out if this is largest list as well
            if num_elements > best_choice_len:
                best_choice = i
                best_choice_len = num_elements


    hwmean_valid = hwmean[inc_list]

    signal_var = np.var(hwmean_valid, axis=0)
    noise_var_onehw = np.var(hwarray[best_choice], axis=0)

    snr = signal_var / noise_var_onehw

    if db:
        return 20*np.log(snr)

    return snr



#!/usr/bin/python
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_DEBUG
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import ReaderTemplate
from chipwhisperer.common.utils import timer
from chipwhisperer.common.utils.parameter import setupSetParam

try:
    from smartcard.CardType import AnyCardType
    from smartcard.CardRequest import CardRequest
    from smartcard.CardConnection import CardConnection
    from smartcard.util import toHexString
except ImportError:
    raise ImportError("Smartcard libraries are missing")


class ReaderPCSC(ReaderTemplate):
    _name = "PC/SC Reader"

    def __init__(self):
        ReaderTemplate.__init__(self)

        self.timeoutTimer = timer.Timer()
        self.timeoutTimer.timeout.connect(self.timeoutFired)
        self.timeoutTimer.setInterval(2000)
        self.getParams().addChildren([
            {'name':'Keep-Alive Interval (off=0)', 'type':'int', 'value':2, 'set':self.setKeepalive}
        ])

    @setupSetParam("Keep-Alive Interval (off=0)")
    def setKeepalive(self, kinterval):
        self.timeoutTimer.setInterval(kinterval*1000)

    def timeoutFired(self):
        self.scserv.connection.connect()

    def sendAPDU(self, cla, ins, p1, p2, txdata=None, rxdatalen=0):
        """Send APDU to SmartCard, get Response"""

        data = [cla,ins, p1, p2]

        if txdata is not None:
            txdatalen = len(txdata)
            data.append(txdatalen)
        else:
            txdatalen = 0

        if (txdata is None) & (rxdatalen == 0):
            data.append(0)

        # Append payload
        if txdata is not None:
            for b in txdata: data.append(b)

        if rxdatalen != 0:
            data.append(rxdatalen)

        response, sw1, sw2 = self.scserv.connection.transmit(data , CardConnection.T0_protocol)

        status = (sw1 << 8) | sw2

        if rxdatalen > 0:
            return (status, response)

        return status

    def con(self, scope=None):
        """Connect to reader. scope parameter is OpenADC/ChipWhisperer hardware, only used to integrated readers"""
        try:
            self.sccard = AnyCardType()
            self.screq = CardRequest(timeout=1, cardType=self.sccard)
            self.scserv = self.screq.waitforcard()

            if not self.timeoutTimer.isActive():
                self.timeoutTimer.start()

            logging.info('SCARD: Connected...')
        except Exception:
            raise Exception("SCARD: Failed to connect...")

    def flush(self):
        """Discard all input buffers"""
        pass

    def reset(self):
        """Reset card & save the ATR"""
        pass

    def getATR(self):
        """Get the ATR from the SmartCard. Reads a saved value, user reset() to actually reset card."""
        pass

    def dis(self):
        self.scserv.connection.disconnect()
        self.timeoutTimer.stop()



#!/usr/bin/python
# HIGHLEVEL_CLASSLOAD_FAIL_FUNC_DEBUG
# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

import hid
from ._base import TargetTemplate


class HIDSPI(object):
    CMDSPI = 0x01
    CMDBOOT = 0xFE

    def findCWSPI(self, VID=0x03EB, PID=0xBAED):
        logging.info('Detecting HID device...')
        self.hdev = hid.device(VID, PID)
        logging.info('Manufacturer: %s' % self.hdev.get_manufacturer_string())
        logging.info('Product: %s' % self.hdev.get_product_string())

    def sendHID(self, cmd, data=[]):
        # Report is 64 bytes. -2 for our bytes
        padding = 64 - 2 - len(data)

        # Generate the message to pass to the stack
        hidmsg = []

        # Report ID is first byte (only 1 report here)
        hidmsg.append(0x00)

        # Command
        hidmsg.append(cmd)

        # Length of Payload (if applicable)
        hidmsg.append(len(data))

        # Payload padded to correct size
        hidmsg = hidmsg + data
        hidmsg = hidmsg + [0] * padding

        # Pass onward
        if self.hdev == None:
            return
        elif self.hdev != 2:
        # else:
            # print "HID Write: ",
            # for i in hidmsg:
            #    print "0x%02x"%i,
            # print ""
            if self.hdev.write(hidmsg) < 0:
                raise IOError("Error calling hdev.write()")

    def sendSPI(self, data):
        self.sendHID(self.CMDSPI, data)
        data = self.hdev.read(65)
        if data[0] & 0x80:
            plen = data[0] & 0x7F
            results = data[1:(1 + plen)]
        else:
            raise IOError("Invalid input HID Report")

        # print results
        return results

    def jumpBootloader(self):
        self.sendHID(self.CMDBOOT)


class ChipWhispererSPI(TargetTemplate):
    _name = "ChipWhisperer SPI"

    def __init__(self):
        TargetTemplate.__init__(self)
        self.hdev = HIDSPI()
        self.keylength = 16
        self.params.addChildren([
            {'name':'Jump to Bootloader', 'type':'action', 'action':self.hdev.jumpBootloader}
        ])

    def setKeyLen(self, klen):
        """ Set key length in BITS """
        self.keylength = klen / 8

    def keyLen(self):
        """ Return key length in BYTES """
        return self.keylength

    def _con(self, scope=None):
        self.hdev.findCWSPI()

    def init(self):
        return

    def setModeEncrypt(self):
        return

    def setModeDecrypt(self):
        return

    def loadEncryptionKey(self, key):
        self.key = key

    def loadInput(self, inputtext):
        self.input = inputtext

    def isDone(self):
        return True

    def readOutput(self):
        output = self.hdev.sendSPI([0] * 32)

        if output[:16] != output[16:]:
            raise IOError("Error, response not repeated: %s" % str(output))

        return output[:16]

    def go(self):
        self.hdev.sendSPI(list(self.input + self.key))

    def checkEncryptionKey(self, kin):
        blen = self.keyLen()

        if len(kin) < blen:
            logging.warning('Padding key...')
            newkey = bytearray(kin)
            newkey += bytearray([0] * (blen - len(kin)))
            return newkey
        elif len(kin) > blen:
            logging.warning('Truncating key...')
            return kin[0:blen]

        return kin



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
from chipwhisperer.common.utils import util
from chipwhisperer.hardware.naeusb.programmer_avr import supported_avr
from chipwhisperer.hardware.naeusb.programmer_xmega import supported_xmega
from chipwhisperer.hardware.naeusb.programmer_stm32fserial import supported_stm32f

from functools import wraps


def save_and_restore_pins(func):
    """Decorator to save and restore pins needed to comunicate and program hardware

        Purpose: to move from changing pins in the background needed to communicate
         with the hardware and leaving them changed (very confusing), to saving the
         pin states before the function is called, changing them during function
         executation and restoring them after the function is done executing
         (less confusing).
        """
    @wraps(func) # updates func_wrapper aatributes to be same
    def func_wrapper(self, *args, **kwargs):

        #If no scope, we don't do any pin magic
        if self.scope is None:
            return func(self, *args, **kwargs)

        pin_setup = {
            'pdic': self.scope.io.pdic,
            'pdid': self.scope.io.pdid,
            'nrst': self.scope.io.nrst,
        }
        logging.debug('Saving pdic, pdid, and nrst pin configuration')
        # setup the pins so that so communication to the target is possible
        # Important: during the execution of func, the pin values may change if
        # the function is related to reprogramming or resetting the device. Example:
        # the stm32f uses the toggling of the nrst and pdic pins for resetting
        # and boot mode setting respectively
        logging.debug('Changing pdic, pdid, and nrst pin configuration')
        if pin_setup['pdic'] != 'high_z':
            self.scope.io.pdic = 'high_z'
        if pin_setup['pdid'] != 'high_z':
            self.scope.io.pdid = 'high_z'
        if pin_setup['nrst'] != 'high_z':
            self.scope.io.nrst = 'high_z'
        try:
            val = func(self, *args, **kwargs)
        finally:
            logging.debug('Restoring pdic, pdid, and nrst pin configuration')
            if self.scope.io.pdic != pin_setup['pdic']:
                self.scope.io.pdic = pin_setup['pdic']
            if self.scope.io.pdid != pin_setup['pdid']:
                self.scope.io.pdid = pin_setup['pdid']
            if self.scope.io.nrst != pin_setup['nrst']:
                self.scope.io.nrst = pin_setup['nrst']
        return val # only returns value when decorating a function with return value
    return func_wrapper



class Programmer(object):
    lastFlashedFile = "unknown"
    _scope = None
    pin_setup = {}

    def __init__(self):
        self.newTextLog = util.Signal()
        self._scope = None

    def open(self):
        pass

    @property
    def scope(self):
        if self._scope:
            return self._scope

        return None

    @scope.setter
    def scope(self, value):
        self._scope = value

    def save_pin_setup(self):
        self.pin_setup['pdic'] = self.scope.io.pdic
        self.pin_setup['pdid'] = self.scope.io.pdid
        self.pin_setup['nrst'] = self.scope.io.nrst

    def restore_pin_setup(self):
        self.scope.io.pdic = self.pin_setup['pdic']
        self.scope.io.pdid = self.pin_setup['pdid']
        self.scope.io.nrst = self.pin_setup['nrst']

    def set_pins(self):
        raise NotImplementedError

    def setUSBInterface(self, iface):
        raise DeprecationWarning('find method now includes what setUSBInterface did')

    def find(self):
        raise NotImplementedError

    def program(self, filename, memtype="flash", verify=True):
        raise NotImplementedError

    def erase(self):
        raise NotImplementedError

    def close(self):
        raise NotImplementedError

    def log(self, text):
        """Logs the text and broadcasts it"""
        logging.info(text)
        self.newTextLog.emit(text)

    def autoProgram(self, hexfile, erase, verify, logfunc, waitfunc):
        raise NotImplementedError


class AVRProgrammer(Programmer):
    def __init__(self):
        self.slow_clock = False
        super(AVRProgrammer, self).__init__()

    @save_and_restore_pins
    def find(self):
        avr = self.scope.scopetype.dev.avr
        sig, chip = avr.find(self.slow_clock)
        if chip is None:
            self.log("AVR: Detected unknown device with signature=%2x %2x %2x" % (sig[0], sig[1], sig[2]))
        else:
            self.log("AVR: Detected device %s" % chip.name)

    @save_and_restore_pins
    def erase(self):
        avr = self.scope.scopetype.dev.avr
        self.log("Erasing Chip")
        avr.eraseChip()

    @save_and_restore_pins
    def autoProgram(self, hexfile, erase, verify, logfunc, waitfunc):
        avr = self.scope.scopetype.dev.avr
        avr.autoProgram(hexfile, erase, verify, logfunc, waitfunc)

    @save_and_restore_pins
    def writeFuse(self, value, lfuse):
        avr = self.scope.scopetype.dev.avr
        avr.writeFuse(value, lfuse)

    @save_and_restore_pins
    def readFuse(self, value):
        avr = self.scope.scopetype.dev.avr
        return avr.readFuse(value)

    @save_and_restore_pins
    def enableSlowClock(self, value):
        if value:
            self.slow_clock = True
        else:
            self.slow_clock = False
        avr = self.scope.scopetype.dev.avr
        avr.enableSlowClock(value)

    @save_and_restore_pins
    def program(self, filename, memtype="flash", verify=True):
        avr = self.scope.scopetype.dev.avr
        Programmer.lastFlashedFile = filename
        avr.program(filename, memtype, verify)

    @save_and_restore_pins
    def autoProgram(self, hexfile, erase, verify, logfunc, waitfunc):
        avr = self.scope.scopetype.dev.avr
        avr.autoProgram(hexfile, erase, verify, logfunc, waitfunc)

    @save_and_restore_pins
    def close(self):
        avr = self.scope.scopetype.dev.avr
        try:
            avr.enableISP(False)
        except AttributeError as e:
            logging.info("AVR programmer: could not disable ISP - USB might be disconnected!")


class XMEGAProgrammer(Programmer):

    def __init__(self):
        super(XMEGAProgrammer, self).__init__()
        self.supported_chips = supported_xmega
        self.xmega = None

    def xmegaprog(self):
        if self.xmega is None:
            xmega = self.scope.scopetype.dev.xmega
        else:
            xmega = self.xmega
        return xmega

    @save_and_restore_pins
    def find(self, xmega=None):
        xmega = self.xmegaprog()
        sig, chip = xmega.find()

        # Print signature of unknown device
        if chip is None:
            self.log("Detected Unknown Chip, sig=%2x %2x %2x" % (sig[0], sig[1], sig[2]))
        else:
             self.log("Detected %s" % chip.name)

    @save_and_restore_pins
    def erase(self, memtype="chip"):
        self.log("Erasing Chip")
        xmega = self.xmegaprog()
        try:
            xmega.erase(memtype)
        except IOError:
            logging.info("Full chip erase timed out. Reinitializing programmer and erasing only application memory")
            self.open()
            self.find()
            xmega.enablePDI(False)
            xmega.enablePDI(True)
            xmega.erase("app")

    @save_and_restore_pins
    def autoProgram(self, hexfile, erase, verify, logfunc, waitfunc):
        xmega = self.scope.scopetype.dev.xmega
        xmega.autoProgram(hexfile, erase, verify, logfunc, waitfunc)

    @save_and_restore_pins
    def program(self, filename, memtype="flash", verify=True):
        Programmer.lastFlashedFile = filename
        xmega = self.xmegaprog()
        xmega.program(filename, memtype, verify)

    @save_and_restore_pins
    def close(self):
        xmega = self.xmegaprog()
        xmega.enablePDI(False)

class STM32FProgrammer(Programmer):

    def __init__(self, small_blocks=False, slow_prog=False, baud=115200):
        super(STM32FProgrammer, self).__init__()
        self.supported_chips = supported_stm32f
        self._baud = baud

        self.slow_speed = slow_prog
        self.small_blocks = small_blocks
        self.stm = None

    def stm32prog(self):

        if self.stm is None:
            stm = self.scope.scopetype.dev.serialstm32f
        else:
            stm = self.stm

        stm.slow_speed = self.slow_speed
        stm.small_blocks = self.small_blocks

        return stm

    @save_and_restore_pins
    def open_and_find(self, log_func=None):
        stm32f = self.stm32prog()
        stm32f.open_port()
        sig, chip = stm32f.find(logfunc=log_func)

        # logging is done at the lower level

    @save_and_restore_pins
    def open(self):
        stm32f = self.stm32prog()
        stm32f.open_port(self._baud)

    @save_and_restore_pins
    def find(self):
        stm32f = self.stm32prog()
        stm32f.scope = self.scope
        sig, chip = stm32f.find()

        # logging is done at the lower level

    @save_and_restore_pins
    def program(self, filename, memtype="flash", verify=True):
        Programmer.lastFlashedFile = filename
        stm32f = self.stm32prog()
        stm32f.scope = self.scope
        stm32f.program(filename, memtype, verify)

    @save_and_restore_pins
    def autoProgram(self, hexfile, erase, verify, logfunc, waitfunc):
        stm32f = self.stm32prog()
        stm32f.scope = self.scope
        stm32f.autoProgram(hexfile, erase, verify, logfunc, waitfunc)

    @save_and_restore_pins
    def erase(self):
        self.log("Erasing Chip")
        stm32f = self.stm32prog()
        stm32f.cmdEraseMemory()

    @save_and_restore_pins
    def close(self):
        stm32f = self.stm32prog()
        stm32f.close_port()
        stm32f.releaseChip()




def CWCalcClkDiv(sysclk, targfreq):
    clkdivider = int((sysclk / (2 * targfreq)) + 1)   
    
    try:
        actual = sysclk / (2 * (clkdivider - 1))
    except ZeroDivisionError:
        actual = 0
    error = (float(actual - targfreq) / targfreq)*100.0        
    
    return (clkdivider, error)
    
    #    desired_freq = 38400 * 3
    #    clk = 30E6
    #    clkdivider = (clk / (2 * desired_freq)) + 1        
    #    self.cwADV.setIOPattern(strToPattern("\n"), clkdiv=clkdivider)


def strToBits(string, startbits=1, stopbits=1, parity="none", guardbits=0):
    """Converts a string into a standard serial pattern"""
    totalpat = []
    for s in string:
        
        try:        
            bp = ord(s)
        except:
            bp = s
            
        if s is None:
            for i in range(0, startbits):
                totalpat.append(1)
                
            for i in range(0, 8):
                totalpat.append(1)
                
            if parity != "none":
                totalpat.append(1)
                
            for i in range(0, stopbits):
                totalpat.append(1)
                
            for i in range(0, guardbits):
                totalpat.append(1)
        else:
                
            #Start bits
            for i in range(0, startbits):
                totalpat.append(0)
             
            onebits = 0
             
            #Following for MSB first:   
            #for i in range(7, -1, -1):
            #Serial protocols LSB first:
            for i in range(0,  8):
                bit = (bp >> i) & 0x01
                if bit:
                    onebits += 1
                totalpat.append(bit)
    
            #Parity bits
            if (onebits % 2) == 0:
                if parity == "even":
                    totalpat.append(0)
                elif parity == "odd":
                    totalpat.append(1)
            else:
                if parity == "even":
                    totalpat.append(1)
                elif parity == "odd":
                    totalpat.append(0)            
    
            #Stop bits
            for i in range(0, stopbits):
                totalpat.append(1)
    
            for i in range(0, guardbits):
                totalpat.append(1)       
                
    return totalpat



# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging
import sys
import traceback
# import chipwhisperer.capture.scopes._qt as openadc_qt
from .. import _qt as openadc_qt
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererFWLoader import CWLite_Loader, CW1200_Loader
from chipwhisperer.capture.scopes.cwhardware.ChipWhispererFWLoader import FWLoaderConfig
from chipwhisperer.common.utils.util import DictType, camel_case_deprecated

try:
    from chipwhisperer.capture.scopes.cwhardware import ChipWhispererLite as CWL
except ImportError:
    CWL = None
    logging.error("Could not import ChipWhispererLite\n" + traceback.format_exc())

try:
    import usb
except ImportError:
    usb = None
    logging.error("Could not import USB\n" + traceback.format_exc())


class OpenADCInterface_NAEUSBChip(object):
    _name = "NewAE USB (CWLite/CW1200)"

    def __init__(self, oadcInstance):
        self.ser = None
        self.dev = None
        self.scope = None
        self.last_id = None

        if (openadc_qt is None) or (usb is None):
            missingInfo = ""
            if openadc_qt is None:
                missingInfo += "openadc.qt "
            if usb is None:
                missingInfo += " usb"
            raise ImportError("Needed imports for ChipWhisperer missing: %s" % missingInfo)
        else:
            self.cwFirmwareConfig = {
                0xACE2:FWLoaderConfig(CWLite_Loader()),
                0xACE3:FWLoaderConfig(CW1200_Loader())
            }
            self.scope = oadcInstance

    def con(self, sn=None):
        if self.ser is None:
            self.dev = CWL.CWLiteUSB()

            try:
                nae_products = [0xACE2, 0xACE3]
                possible_sn = self.dev.get_possible_devices(nae_products)
                serial_numbers = []
                if len(possible_sn) > 1:
                    #Update list...
                    if sn is None:
                        snlist = DictType({'Select Device to Connect':None})
                        for d in possible_sn:
                            snlist[str(d['sn']) + " (" + str(d['product']) + ")"] = d['sn']
                            serial_numbers.append("sn = {} ({})".format(str(d['sn']), str(d['product'])))
                            pass
                        raise Warning("Multiple ChipWhisperers detected. Please specify device from the following list using cw.scope(sn=<SN>): \n{}".format(serial_numbers))
                else:
                    pass
                    #if possible_sn[0]['sn'] !=
                    #sn = None
                found_id = self.dev.con(idProduct=nae_products, serial_number=sn)
            except (IOError, ValueError):
                raise Warning('Could not connect to "%s". It may have been disconnected, is in an error state, or is being used by another tool.' % self.getName())

            if found_id != self.last_id:
                logging.info("Detected ChipWhisperer with USB ID %x - switching firmware loader" % found_id)
            self.last_id = found_id

            self.getFWConfig().setInterface(self.dev.fpga)
            try:
                self.getFWConfig().loadRequired()
            except:
                self.dev.dis()
                self.dev.usbdev().close()
                raise
            self.ser = self.dev.usbdev()

        try:
            self.scope.con(self.ser)
            logging.info('OpenADC Found, Connecting')
        except IOError as e:
            exctype, value = sys.exc_info()[:2]
            raise IOError("OpenADC: " + (str(exctype) + str(value)))

    def dis(self):
        if self.ser is not None:
            self.getFWConfig().setInterface(None)
            self.scope.close()
            self.ser.close()
            self.ser = None
        if self.dev is not None:
            self.dev.dis()
            self.dev = None

    def __del__(self):
        if self.ser is not None:
            self.ser.close()

    def getFWConfig(self):
        try:
            return self.cwFirmwareConfig[self.last_id]
        except KeyError as e:
            return FWLoaderConfig(CWLite_Loader())

    def get_name(self):
        return self._name

    getName = camel_case_deprecated(get_name)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import random
from chipwhisperer.common.utils import util
from ._base import AcqKeyTextPattern_Base
from chipwhisperer.common.utils.parameter import setupSetParam


class AcqKeyTextPattern_DPA(AcqKeyTextPattern_Base):
    _name = "DPA Helper"

    def __init__(self, target=None):
        AcqKeyTextPattern_Base.__init__(self)
        self.initmask = 'FF 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00'
        self.initkey = '2B 7E 15 16 28 AE D2 A6 AB F7 15 88 09 CF 4F 3C'
        self._key = util.hexStrToByteArray(self.initkey)
        self._mask = util.hexStrToByteArray(self.initmask)

        self.getParams().addChildren([
            {'name':'Fixed Encryption Key', 'key':'initkey', 'type':'str', 'get':self.getInitialKey, 'set':self.setInitialKey, 'visible':True},
            {'name':'Plaintext Mask', 'key':'initmask', 'type':'str', 'get':self.getInitialMask, 'set':self.setInitialMask, 'visible':True},
        ])
        self.setTarget(target)

    def getInitialKey(self):
        return " ".join(["%02X"%b for b in self._key])

    @setupSetParam("Fixed Encryption Key")
    def setInitialKey(self, initialKey, binaryKey=False):
        if initialKey:
            if binaryKey:
                keyStr = ''
                for s in initialKey:
                    keyStr += '%02x ' % s
                self._key = bytearray(initialKey)
            else:
                keyStr = initialKey
                self._key = util.hexStrToByteArray(initialKey)

            self.initkey = keyStr

    def getInitialMask(self):
        return " ".join(["%02X" % b for b in self._mask])

    @setupSetParam("Fixed Plaintext")
    def setInitialMask(self, initialMask, binaryMask=False):
        if initialMask:
            if binaryMask:
                maskStr = ''
                for s in initialMask:
                    maskStr += '%02x ' % s
                self._mask = bytearray(initialMask)
            else:
                maskStr = initialMask
                self._mask = util.hexStrToByteArray(initialMask)

            self.initmask = maskStr

    def initPair(self, maxtraces):
        pass

    def new_pair(self):
        randDPA = random.randint(0, 1)
        if randDPA == 0:
            DPAByte = 0
        else:
            DPAByte = 0xFF
        self._textin = bytearray(self.textLen())
        for i in range(0, self.textLen()):
            randbyte = random.randint(0, 255)
            self._textin[i] = (randbyte & ~self._mask[i]) | (DPAByte & self._mask[i])

        # Check pair works with target
        self.validateKey()
        self.validateText()

        return self._key, self._textin

    newPair = new_pair



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
#
#    Date         Author                Changes
#    -----------  --------------------  -----------------------------------
#    20-Jun-2016  marc                  Added "bandstop" filter option

from ._base import PreprocessingBase
from scipy import signal
from chipwhisperer.common.utils.parameter import setupSetParam

class Filter(PreprocessingBase):
    """
    Generic filter, pulls in from SciPy for doing the actual filtering of things
    """
    _name = "Digital Filter"
    _description = "Frequency specific filter."

    valid_types = ["low", "high", "bandpass", "bandstop"]

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)

        self._type = "low"
        self._freq1 = 0.1
        self._freq2 = 0.8
        self._order = 5

        self.getParams().addChildren([
            {'name':'Form', 'key':'form', 'type':'list', 'values':{"Butterworth":"sp.signal.butter"}, 'default':"sp.signal.butter", 'value':"sp.signal.butter"},
            {'name':'Type', 'key':'type', 'type':'list', 'values':self.valid_types, 'get':self._getType, 'set':self._setType},
            {'name':'Critical Freq #1 (0-1)', 'key':'freq1', 'type':'float', 'limits':(0, 1), 'step':0.05, 'get':self._getFreq1, 'set':self._setFreq1},
            {'name':'Critical Freq #2 (0-1)', 'key':'freq2', 'type':'float', 'limits':(0, 1), 'step':0.05, 'get':self._getFreq2, 'set':self._setFreq2},
            {'name':'Order', 'key':'order', 'type':'int', 'limits':(1, 32), 'get':self._getOrder, 'set':self._setOrder},
        ])
        self._updateFilterParams()

    @property
    def form(self):
        """The filter implementation. Only supported option is Butterworth, so
        this parameter is read-only.
        """
        return "butterworth"

    def _getType(self):
        return self._type

    @setupSetParam("Type")
    def _setType(self, type):
        self._type = type
        self._updateFilterParams()

    @property
    def type(self):
        """The type of filter. Options are:
        - "low": Low-pass filter (pass signals below critical freq)
        - "high": High-pass filter (pass signals above critical freq)
        - "bandpass": Band-pass filter (pass signals between critical freqs)
        - "bandstop": Band-stop (or notch) filter (stop signals between freqs)

        Setter raises ValueError if filter type isn't one of these
        """
        return self._getType()

    @type.setter
    def type(self, type):
        if type not in self.valid_types:
            raise ValueError("Unrecognized filter mode; expected one of %s" % self.valid_types, type)
        self._setType(type)

    def _getFreq1(self):
        return self._freq1

    @setupSetParam("Critical Freq #1 (0-1)")
    def _setFreq1(self, freq):
        self._freq1 = freq
        self._updateFilterParams()

    @property
    def freq1(self):
        """The first critical frequency as a float in the range [0, 1].

        This is the sole cutoff frequency for low/high pass filters and the
        bottom frequency for bandpass/bandstop filters.

        Setter raises TypeError if value not a float.
        """
        return self._getFreq1()

    @freq1.setter
    def freq1(self, freq):
        if not isinstance(freq, float):
            raise TypeError("Expected float; got %s" % type(freq), freq)
        self._setFreq1(freq)

    def _getFreq2(self):
        return self._freq2

    @setupSetParam("Critical Freq #2 (0-1)")
    def _setFreq2(self, freq):
        self._freq2 = freq
        self._updateFilterParams()

    @property
    def freq2(self):
        """The second critical frequency as a float in the range [0, 1].

        For bandpass/bandstop filters, this is the upper critical frequency.
        For low/high pass filters, this value is not used.

        Setter raises TypeError if value not a float.
        """
        return self._getFreq2()

    @freq2.setter
    def freq2(self, freq):
        if not isinstance(freq, float):
            raise TypeError("Expected float; got %s" % type(freq), freq)
        self._setFreq2(freq)

    def _getOrder(self):
        return self._order

    @setupSetParam("Order")
    def _setOrder(self, order):
        self._order = order
        self._updateFilterParams()

    @property
    def order(self):
        """The order of the digital filter.

        Setter raises TypeError if value not an integer.
        """
        return self._getOrder()

    @order.setter
    def order(self, order):
        if not isinstance(order, int):
            raise TypeError("Expected int; got %s" % type(order), order)
        self._setOrder(order)

    def _setFilterForm(self, filtform=signal.butter):
        """Set the filter type in object"""
        self.filterForm = filtform

    def _setFilterParams(self, form='low', freq=0.8, order=5):
        self.b, self.a = self.filterForm(order, freq, form)

    def _updateFilterParams(self):
        if self._type in ("bandpass", "bandstop"):
            freqs = (self._freq1, self._freq2)
        else:
            freqs = self._freq1
        self._setFilterForm()
        self._setFilterParams(self._type, freqs, self._order)

    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None
            return signal.lfilter(self.b, self.a, trace)
        else:
            return self._traceSource.getTrace(n)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import os

class tracereader_native:
    def __init__(self):
        self.numTrace = None
        self.numPoint = None
        self.textints = None
        self.textouts = None
        self.knownkey = None
        self.directory = None
        self.tracesSaved = False
        self.tracedtype = None

    def copyTo(self, srcTraces=None):
        self.numTrace = srcTraces.numTraces()
        self.numPoint = srcTraces.numPoints()
        self.knownkey = srcTraces.knownkey

        self.textins = np.zeros([self.numTrace, 16], dtype=np.uint8)
        for n in range(0, self.numTrace):
            tin = srcTraces.textins[n]
            self.textins[n] = list(map(int, tin, [16]*len(tin)))
        

        self.textouts = np.zeros([self.numTrace, 16], dtype=np.uint8)
        for n in range(0, self.numTrace):
            tout = srcTraces.textouts[n]
            self.textouts[n] = list(map(int, tout, [16]*len(tout)))

        if srcTraces.tracedtype:
            userdtype = srcTraces.tracedtype
        else:
            userdtype = np.float
            
        self.traces = np.array(srcTraces.traces, dtype=userdtype)

        #Traces copied in means not saved
        self.tracesSaved = False
        
    def loadAllTraces(self, directory=None, prefix=""):
        self.traces = np.load(directory + "/%straces.npy"%prefix)
        self.textins = np.load(directory + "/%stextin.npy"%prefix)
        self.textouts = np.load(directory + "/%stextout.npy"%prefix)
        self.knownkey = np.load(directory + "/%sknownkey.npy"%prefix)

        self.numTrace = self.traces.shape[0]
        self.numPoint = self.traces.shape[1]

        #Traces loaded means saved
        self.tracesSaved = True
        return

    def saveAllTraces(self, directory, prefix=""):
        np.save(directory + "/%straces.npy"%prefix, self.traces)
        np.save(directory + "/%stextin.npy"%prefix, self.textins)
        np.save(directory + "/%stextout.npy"%prefix, self.textouts)
        np.save(directory + "/%sknownkey.npy"%prefix, self.knownkey)
        self.tracesSaved = True

    def numPoints(self):
        return self.numPoint

    def numTraces(self):
        return self.numTrace

    def getTrace(self, n):
        data = self.traces[n]

        #Following line will normalize all traces relative to each
        #other by mean & standard deviation
        #data = (data - np.mean(data)) / np.std(data)
        
        return data

    def getTextin(self, n):
        return self.textins[n]

    def getTextout(self, n):
        return self.textouts[n]

    def getKnownKey(self):
        return self.knownkey
    
    
    



from chipwhisperer.analyzer.attacks.models import AES128_8bit as aes128_leakage
from chipwhisperer.analyzer.attacks.models.AES128_8bit import AES128_8bit as AES128_8bit
import textwrap
from chipwhisperer.analyzer.attacks.models.AES128_8bit import AESLeakageHelper


class EightBitAES128LeakageModels:
    """Contains the available leakage models for 8 bit AES128.

    To use a leakage model during a correlation power analysis attack
    see :func:`cpa <chipwhisperer.analyzer.cpa>`.

    :data:`leakage_models <chipwhisperer.analyzer.leakage_models>` is an instance of
    this class.
    """

    def new_model(self, model):
        """Makes a new leakage model for use with analyzer

        Usage::

            class AES256_Round13_Model(cwa.AESLeakageHelper):
                def leakage(self, pt, ct, guess, bnum):
                    # You must put YOUR recovered 14th round key here.
                    # This example may not be accurate!
                    calc_round_key = [0xea, 0x79, 0x79, 0x20, 0xc8, 0x71,
                                      0x44, 0x7d, 0x46, 0x62, 0x5f, 0x51,
                                      0x85, 0xc1, 0x3b, 0xcb]
                    xored = [calc_round_key[i] ^ pt[i] for i in range(0, 16)]
                    block = xored
                    block = self.inv_shiftrows(block)
                    block = self.inv_subbytes(block)
                    block = self.inv_mixcolumns(block)
                    block = self.inv_shiftrows(block)
                    result = block
                    return self.inv_sbox((result[bnum] ^ guess[bnum]))

            leak_model = cwa.leakage_models.new_model(AES256_Round13_Model)

        Args:
            model (AESLeakageHelper): New leakage model to create

        Returns:
            Leakage model made from model

        """
        return AES128_8bit(model)

    @property
    def plaintext_key_xor(self):
        """Plain text key XOR."""
        return AES128_8bit(aes128_leakage.PtKey_XOR)

    @property
    def sbox_output(self):
        """SBox Output."""
        return AES128_8bit(aes128_leakage.SBox_output)

    @property
    def inverse_sbox_output(self):
        """Inverse Sbox output."""
        return AES128_8bit(aes128_leakage.InvSBox_output)

    @property
    def last_round_state(self):
        """Last round state."""
        return AES128_8bit(aes128_leakage.LastroundHW)

    @property
    def after_key_mix(self):
        """After key mix operation."""
        return AES128_8bit(aes128_leakage.AfterKeyMixin)

    @property
    def mix_columns_output(self):
        """Output of the mix columns operation."""
        return AES128_8bit(aes128_leakage.Mixcolumns_output)

    @property
    def shift_columns_output(self):
        """Output of the shift columns operation."""
        return AES128_8bit(aes128_leakage.ShiftColumns_output)

    @property
    def last_round_state_diff(self):
        """Last round state using hamming distance."""
        return AES128_8bit(aes128_leakage.LastroundStateDiff)

    @property
    def last_round_state_diff_alternate(self):
        """Last round state using hamming distance, alternate."""
        return AES128_8bit(aes128_leakage.LastroundStateDiffAlternate)

    @property
    def sbox_in_out_diff(self):
        """Hamming distance between SBox input and output."""
        return AES128_8bit(aes128_leakage.SBoxInOutDiff)

    @property
    def sbox_input_successive(self):
        """Successive Sbox input."""
        return AES128_8bit(aes128_leakage.SBoxInputSuccessive)

    @property
    def sbox_output_successive(self):
        """Successive SBox output."""
        return AES128_8bit(aes128_leakage.SBoxOutputSuccessive)

    @property
    def round_1_2_state_diff_text(self):
        """Hamming distance between round 1 and 2 state using plain text."""
        return AES128_8bit(aes128_leakage.Round1Round2StateDiff_Text)

    @property
    def round_1_2_state_diff_key_mix(self):
        """Hamming distance between round 1 and 2 state during key mix
        operation.
        """
        return AES128_8bit(aes128_leakage.Round1Round2StateDiff_KeyMix)

    @property
    def round_1_2_state_diff_sbox(self):
        """Hamming distance between round 1 and 2 state during sbox lookup
        operation.
        """
        return AES128_8bit(aes128_leakage.Round1Round2StateDiff_SBox)

    def __str__(self):
        """Structured glossary of all the different leakage models.

        The glossary includes the name of the leakage model and the
        description (doc string).
        """
        include = [
            'plaintext_key_xor',
            'sbox_output',
            'inverse_sbox_output',
            'last_round_state',
            'after_key_mix',
            'mix_columns_output',
            'shift_columns_output',
            'last_round_state_diff',
            'last_round_state_diff_alternate',
            'sbox_in_out_diff',
            'sbox_input_successive',
            'sbox_output_successive',
            'round_1_2_state_diff_text',
            'round_1_2_state_diff_key_mix',
            'round_1_2_state_diff_sbox'
        ]
        models = [x for x in dir(self) if x in include]
        doc_strings = [getattr(self.__class__, x).__doc__ for x in models]

        items = []
        for model, doc_string in zip(models, doc_strings):
            if doc_string is None:
                items.append(model + ':')
            else:
                dedented_string = textwrap.dedent(doc_string)
                wrapped_string = '\n'.join(textwrap.wrap(dedented_string, 70))
                indented_doc_string = textwrap.indent(wrapped_string, ' ' * 2)
                items.append(':\n'.join([model, indented_doc_string]))

        return '\n\n'.join(items)




#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

from .base import ModelsBase
from chipwhisperer.analyzer.attacks.models.aes.funcs import sbox, inv_sbox


class AES(object):

    # Rijndael S-box
    sbox = [0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67,
            0x2b, 0xfe, 0xd7, 0xab, 0x76, 0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59,
            0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0, 0xb7,
            0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1,
            0x71, 0xd8, 0x31, 0x15, 0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05,
            0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75, 0x09, 0x83,
            0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29,
            0xe3, 0x2f, 0x84, 0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b,
            0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf, 0xd0, 0xef, 0xaa,
            0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c,
            0x9f, 0xa8, 0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc,
            0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2, 0xcd, 0x0c, 0x13, 0xec,
            0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19,
            0x73, 0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee,
            0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb, 0xe0, 0x32, 0x3a, 0x0a, 0x49,
            0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
            0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4,
            0xea, 0x65, 0x7a, 0xae, 0x08, 0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6,
            0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a, 0x70,
            0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9,
            0x86, 0xc1, 0x1d, 0x9e, 0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e,
            0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf, 0x8c, 0xa1,
            0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0,
            0x54, 0xbb, 0x16]

    # Rijndael Inverted S-box
    rsbox = [0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3,
            0x9e, 0x81, 0xf3, 0xd7, 0xfb , 0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f,
            0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb , 0x54,
            0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b,
            0x42, 0xfa, 0xc3, 0x4e , 0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24,
            0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25 , 0x72, 0xf8,
            0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d,
            0x65, 0xb6, 0x92 , 0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda,
            0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84 , 0x90, 0xd8, 0xab,
            0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3,
            0x45, 0x06 , 0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1,
            0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b , 0x3a, 0x91, 0x11, 0x41,
            0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6,
            0x73 , 0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9,
            0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e , 0x47, 0xf1, 0x1a, 0x71, 0x1d,
            0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b ,
            0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0,
            0xfe, 0x78, 0xcd, 0x5a, 0xf4 , 0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07,
            0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f , 0x60,
            0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f,
            0x93, 0xc9, 0x9c, 0xef , 0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5,
            0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61 , 0x17, 0x2b,
            0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55,
            0x21, 0x0c, 0x7d]

    def getSBoxValue(self, num):
        """Retrieves a given S-Box Value"""
        return self.sbox[num]

    def getSBoxInvert(self, num):
        """Retrieves a given Inverted S-Box Value"""
        return self.rsbox[num]

    def galois_multiplication(self, a, b):
        """Galois multiplication of 8 bit characters a and b."""
        p = 0
        for counter in range(8):
            if b & 1: p ^= a
            hi_bit_set = a & 0x80
            a <<= 1
            # keep a 8 bit
            a &= 0xFF
            if hi_bit_set:
                a ^= 0x1b
            b >>= 1
        return p

    #
    # substitute all the values from the state with the value in the SBox
    # using the state value as index for the SBox
    #
    def subBytes(self, state, isInv):
        if isInv: getter = self.getSBoxInvert
        else: getter = self.getSBoxValue
        for i in range(16): state[i] = getter(state[i])
        return state

    # iterate over the 4 rows and call shiftRow() with that row
    def shiftRows(self, state, isInv):
        for i in range(4):
            state = self.shiftRow(state, i * 4, i, isInv)
        return state

    # each iteration shifts the row to the left by 1
    def shiftRow(self, state, statePointer, nbr, isInv):
        for i in range(nbr):
            if isInv:
                state[statePointer:statePointer + 4] = \
                        state[statePointer + 3:statePointer + 4] + \
                        state[statePointer:statePointer + 3]
            else:
                state[statePointer:statePointer + 4] = \
                        state[statePointer + 1:statePointer + 4] + \
                        state[statePointer:statePointer + 1]
        return state

    # galois multiplication of the 4x4 matrix
    def mixColumns(self, state, isInv):
        # iterate over the 4 columns
        for i in range(4):
            # construct one column by slicing over the 4 rows
            column = state[i:i + 16:4]
            # apply the mixColumn on one column
            column = self.mixColumn(column, isInv)
            # put the values back into the state
            state[i:i + 16:4] = column

        return state

    # galois multiplication of 1 column of the 4x4 matrix
    def mixColumn(self, column, isInv):
        if isInv: mult = [14, 9, 13, 11]
        else: mult = [2, 1, 1, 3]
        cpy = list(column)
        g = self.galois_multiplication

        column[0] = g(cpy[0], mult[0]) ^ g(cpy[3], mult[1]) ^ \
                    g(cpy[2], mult[2]) ^ g(cpy[1], mult[3])
        column[1] = g(cpy[1], mult[0]) ^ g(cpy[0], mult[1]) ^ \
                    g(cpy[3], mult[2]) ^ g(cpy[2], mult[3])
        column[2] = g(cpy[2], mult[0]) ^ g(cpy[1], mult[1]) ^ \
                    g(cpy[0], mult[2]) ^ g(cpy[3], mult[3])
        column[3] = g(cpy[3], mult[0]) ^ g(cpy[2], mult[1]) ^ \
                    g(cpy[1], mult[2]) ^ g(cpy[0], mult[3])
        return column

    def mapin(self, iput):
        block = [0] * 16
        for i in range(4):
            # iterate over the rows
            for j in range(4):
                block[(i + (j * 4))] = iput[(i * 4) + j]
        return block

    def mapout(self, block):

        output = [0] * 16
        # unmap the block again into the output
        for k in range(4):
                    # iterate over the rows
                    for l in range(4):
                        output[(k * 4) + l] = block[(k + (l * 4))]
        return output


class AES256_8bit(ModelsBase):
    SHIFT = []

    INVSHIFT = [0, 5, 10, 15, 4, 9, 14, 3, 8, 13, 2, 7, 12, 1, 6, 11]

    def __init__(self, model=None):
        ModelsBase.__init__(self, 32, 256, model=model)

    def xtime(self, a):
        """xtime operation"""
        a %= 0x100
        b = 0
        if a & 0x80:
            b = 0x1b
        a <<= 1
        a &= 0xff
        a ^= b
        return a

    def HypHW(self, pt, ct, key, bnum):
        """Given either plaintext or ciphertext (not both) + a key guess, return hypothetical hamming weight of result"""
        if pt != None:
            return self.HW[sbox(pt[bnum] ^ key)]
        elif ct != None:
            knownkey = [0xae, 0x83, 0xc1, 0xa5, 0x6b, 0xcb, 0xc6, 0x46, 0x55, 0xa3, 0xbf, 0x8d, 0x58, 0xfa, 0x20, 0x6d]
            a = AES()
            xored = [knownkey[i] ^ ct[i] for i in range(0, 16)]
            block = a.mapin(xored)
            block = a.shiftRows(block, True)
            block = a.subBytes(block, True)
            block = a.mixColumns(block, True)
            block = a.shiftRows(block, True)
            result = a.mapout(block)
            return self.HW[inv_sbox((result[bnum] ^ key))]
        else:
            raise ValueError("Must specify PT or CT")

    def HypHWXtime(self, pt, keyguess, numguess, keyknown, bnumknown):
        """Given plaintext + a subkey guess + a known subkey + subkey numbers return xtime result"""
        a = sbox(pt[numguess] ^ keyguess)
        b = sbox(pt[bnumknown] ^ keyknown)
        raise ValueError("Should this be HW instead of just xtime()???")
        return self.HW[xtime(a^b)]

    def HypHD(self, pt, ct, key, bnum):
        """Given either plaintext or ciphertext (not both) + a key guess, return hypothetical hamming distance of result"""
        #Get output
        if pt != None:
            #TODO: This does't work too well, need to fix
            st2 = sbox(pt[bnum] ^ key)
            st1 = pt[bnum]
            return self.HW[st1 ^ st2]
        elif ct != None:
            st10 = ct[self.INVSHIFT[bnum]]
            st9 = [ct[bnum] ^ key]
            return self.HW[st9 ^ st10]
        else:
            raise ValueError("Must specify PT or CT")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._base import ProtocolTemplate


class ProtocolSASEBOWCardOS(ProtocolTemplate):
    _name = "SASEBO-W SmartCard OS"

    def loadEncryptionKey(self, key):
        if len(key) != 16:
            raise ValueError("Encryption key != 16 bytes??")
        status = self.hw.sendAPDU(0x80, 0x12, 0x00, 0x00, key)
        self.key = key
        #print status

    def loadInput(self, inputtext):
        self.input = inputtext

    def go(self):
        if len(self.input) != 16:
            raise ValueError("Plaintext != 16 bytes??")
        status = self.hw.sendAPDU(0x80, 0x04, 0x04, 0x00, self.input)
        #print status

    def readOutput(self):
        (resp, pay) = self.hw.sendAPDU(0x80, 0xC0, 0x00, 0x00, rxdatalen=16)
        #print resp
        return pay



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

from ._base import ProtocolTemplate
from chipwhisperer.capture.targets.smartcard_readers import chipwhisperer_ser


class ProtocolDPAv42(ProtocolTemplate):
    _name = "DPAContestv4"

    def loadEncryptionKey(self, key):
        if len(key) != 16:
            raise ValueError("Encryption key != 16 bytes??")
        self.key = key

    def loadInput(self, inputtext):
        if len(inputtext) != 16:
            raise ValueError("Plaintext != 16 bytes??")
        self.input = inputtext

    def go(self):
        if not isinstance(self.hw, chipwhisperer_ser.ReaderChipWhispererSER):
            raise IOError("ERROR: DPAContestv4 only works with ChipWhisperer-SER Reader")

        data = []
        data.extend(self.key)
        data.extend(self.input)

        self.hw.ser.flush()
        self.hw.ser.write([0x80, 0xC0, 0x00, 0x00, 32])
        self.hw.waitEcho([0x80, 0xC0, 0x00, 0x00, 32])
        self.hw.ser.write(data)
        self.hw.waitEcho(data)

        #Ack
        ack = self.hw.ser.read(1)
        #print "%02x"%ord(ack[0])

        #Random Block
        rblock = self.hw.ser.read(16)
        rblock = [ord(t) for t in rblock]

        #Ack
        ack = self.hw.ser.read(1)

        #Key + Message sent back
        self.hw.waitEcho(data)

        #Ack - shuffle done
        ack = self.hw.ser.read(1)

        #Receive shuffle blocks back
        suffle0 = self.hw.ser.read(16)
        suffle0 = [ord(t) for t in suffle0]

        suffle1 = self.hw.ser.read(16)
        suffle1 = [ord(t) for t in suffle1]

        #Ack - encryption starting now
        ack = self.hw.ser.read(1)

        #Ack - encryption done now
        ack = self.hw.ser.read(1)

        result = self.hw.ser.read(16)
        result = [ord(t) for t in result]

        self.textout = result

    def readOutput(self):
        return self.textout



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Authors: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.

from collections import OrderedDict
import inspect

from chipwhisperer.analyzer.attacks.models.aes.funcs import sbox, inv_sbox, subbytes, inv_subbytes, mixcolumns, inv_mixcolumns, shiftrows, inv_shiftrows

from .base import ModelsBase
from chipwhisperer.analyzer.attacks.models.aes.key_schedule import key_schedule_rounds
from chipwhisperer.common.utils.util import camel_case_deprecated

class AESLeakageHelper(object):

    #Name of AES Model
    name = 'AES Leakage Model (unnamed)'

    #c model enumeration value, if a C model exists for this device
    c_model_enum_value = None
    c_model_enum_name = None

    INVSHIFT_undo = [0, 5, 10, 15, 4, 9, 14, 3, 8, 13, 2, 7, 12, 1, 6, 11]

    def sbox(self, data):
        """Helper function: performs AES sbox on single byte"""
        return sbox(data)

    def inv_sbox(self, data):
        """Helper function: performs AES inv-sbox on single byte"""
        return inv_sbox(data)

    def subbytes(self, state):
        """Helper function: performs AES sbox on all bytes of state"""
        return subbytes(state)

    def inv_subbytes(self, state):
        """Helper function: performs AES inv-sbox on all bytes of state"""
        return inv_subbytes(state)

    def mixcolumns(self, state):
        """Helper function: performs AES mixcols on all bytes of state"""
        return mixcolumns(state)

    def inv_mixcolumns(self, state):
        """Helper function: performs AES inv-mixcols on all bytes of state"""
        return inv_mixcolumns(state)

    def shiftrows(self, state):
        """Helper function: performs AES shiftrows on all bytes of state"""
        return shiftrows(state)

    def inv_shiftrows(self, state):
        """Helper function: performs AES inv-shiftrows on all bytes of state"""
        return inv_shiftrows(state)

    def key_schedule_rounds(self, inputkey, inputround, desiredround):
        """Helper function: takes AES key from one round to another round-eky """
        return key_schedule_rounds(inputkey, inputround, desiredround)

    def xtime(self, a):
        """Helper function: xtime operation (normally used in software AES)"""
        a %= 0x100
        b = 0
        if a & 0x80:
            b = 0x1b
        a <<= 1
        a &= 0xff
        a ^= b
        return a

    def leakage(self, pt, ct, key, bnum):
        """
        Override this function with specific leakage function (S-Box output, HD, etc).

        Args:
            pt: 16-byte plain-text input
            ct: 16-byte cipher-text output.
            key: 16-byte AES key - byte 'bnum' may be a GUESS if key is known. Rest of bytes may/may not be valid too.
            bnum: Byte number we are trying to attack.

        Returns:
            Value that will be presented on the 8-bit bus. Leakage model (such as HW) will map this to leakage itself.
        """
        raise NotImplementedError("ASKLeakageHelper does not implement leakage")

class PtKey_XOR(AESLeakageHelper):
    name = 'HW: AddRoundKey Output, First Round (Enc)'
    def leakage(self, pt, ct, key, bnum):
        return pt[bnum] ^ key[bnum]

class SBox_output(AESLeakageHelper):
    name = 'HW: AES SBox Output, First Round (Enc)'
    c_model_enum_value = 1
    c_model_enum_name = 'LEAK_HW_SBOXOUT_FIRSTROUND'
    def leakage(self, pt, ct, key, bnum):
        return self.sbox(pt[bnum] ^ key[bnum])

class InvSBox_output(AESLeakageHelper):
    name = 'HW: AES Inv SBox Output, First Round (Dec)'
    c_model_enum_value = 6
    c_model_enum_name = 'LEAK_HW_INVSBOXOUT_FIRSTROUND'
    def leakage(self, pt, ct, key, bnum):
        return self.inv_sbox(pt[bnum] ^ key[bnum])

class LastroundHW(AESLeakageHelper):
    name = 'HW: AES Last-Round State'
    def leakage(self, pt, ct, key, bnum):
        # HD Leakage of AES State between 9th and 10th Round
        # Used to break SASEBO-GII / SAKURA-G
        st10 = ct[self.INVSHIFT_undo[bnum]]
        st9 = inv_sbox(ct[bnum] ^ key[bnum])
        return st9

    def processKnownKey(self, inpkey):
        return key_schedule_rounds(inpkey, 0, 10)


class LastroundStateDiff(AESLeakageHelper):
    name = 'HD: AES Last-Round State'
    c_model_enum_value = 2
    c_model_enum_name = 'LEAK_HD_LASTROUND_STATE'
    def leakage(self, pt, ct, key, bnum):
        # HD Leakage of AES State between 9th and 10th Round
        # Used to break SASEBO-GII / SAKURA-G
        st10 = ct[self.INVSHIFT_undo[bnum]]
        st9 = inv_sbox(ct[bnum] ^ key[bnum])
        return (st9 ^ st10)

    def processKnownKey(self, inpkey):
        return key_schedule_rounds(inpkey, 0, 10)

class LastroundStateDiffAlternate(AESLeakageHelper):
    name = 'HD: AES Last-Round State Alternate'
    def leakage(self, pt, ct, key, bnum):
        # Alternate leakage
        st10 = ct[bnum]
        st9 = inv_sbox(ct[bnum] ^ key[bnum])
        return (st9 ^ st10)

    def processKnownKey(self, inpkey):
        k = key_schedule_rounds(inpkey, 0, 10)
        k = self.shiftrows(k)
        return k

class SBoxInOutDiff(AESLeakageHelper):
    name = 'HD: AES SBox Input to Output'
    c_model_enum_value = 3
    c_model_enum_name = 'LEAK_HD_SBOX_IN_OUT'
    def leakage(self, pt, ct, key, bnum):
        st1 = pt[bnum] ^ key[bnum]
        st2 = self.sbox(st1)
        return st1 ^ st2

class SBoxInputSuccessive(AESLeakageHelper):
    name = 'HD: AES SBox Input i to i+1'
    c_model_enum_name = 4
    c_model_enum_name = 'LEAK_HD_SBOX_IN_SUCCESSIVE'
    def leakage(self, pt, ct, key, bnum):
        st1 = pt[bnum] ^ key[bnum]
        if bnum > 0:
            try:
                st2 = pt[bnum - 1] ^ key[bnum-1]
            except IndexError:
                raise ValueError("Successive requires known key")
        else:
            st2 = 0
        return st1 ^ st2

class SBoxOutputSuccessive(AESLeakageHelper):
    name = 'HD: AES SBox Output i to i+1'
    c_model_enum_value = 5
    c_model_enum_name = 'LEAK_HD_SBOX_OUT_SUCCESSIVE'
    def leakage(self, pt, ct, key, bnum):
        st1 = self.sbox(pt[bnum] ^ key[bnum])
        if bnum > 0:
            try:
                st2 = self.sbox(pt[bnum - 1] ^ key[bnum-1])
            except IndexError:
                raise ValueError("Successive requires known key")
        else:
            st2 = 0
        return st1 ^ st2

class AfterKeyMixin(AESLeakageHelper):
    name = 'HW: AES After Key/PT Addition'
    def leakage(self, pt, ct, key, bnum):
        return pt[bnum] ^ key[bnum]

class Mixcolumns_output(AESLeakageHelper):
    name = 'HW: AES Mixcolumns Output'
    #This is mostly a nonsense leakage model for now, but added for completeness
    def leakage(self, pt, ct, key, bnum):
        state = [pt[i] ^ key[i] for i in range(0, 16)]
        state = self.subbytes(state)
        state = self.shiftrows(state)
        state = self.mixcolumns(state)
        return state[bnum]

class ShiftColumns_output(AESLeakageHelper):
    name = 'HW: AES ShiftColumns Output'
    def leakage(self, pt, ct, key, bnum):
        state = [pt[i] ^ key[i] for i in range(0, 16)]
        state = self.subbytes(state)
        state = self.shiftrows(state)
        return state[bnum]

class Round1Round2StateDiff_Text(AESLeakageHelper):
    name = 'HD: AES Round1/Round2 State diff for text'
    def leakage(self, pt, ct, key, bnum):
        state1 = pt[:]
        state = [pt[i] ^ key[i] for i in range(0, 16)]
        state = self.subbytes(state)
        state = self.shiftrows(state)
        state = self.mixcolumns(state)
        return state[bnum] ^ state1[bnum]

class Round1Round2StateDiff_KeyMix(AESLeakageHelper):
    name = 'HD: AES Round1/Round2 State diff for key addition'
    def leakage(self, pt, ct, key, bnum):
        state = [pt[i] ^ key[i] for i in range(0, 16)]
        state1 = state[:]
        state = self.subbytes(state)
        state = self.shiftrows(state)
        state = self.mixcolumns(state)

        key2 = self.key_schedule_rounds(key, 0, 1)
        state = [state[i] ^ key2[i] for i in range(0, 16)]

        return state[bnum] ^ state1[bnum]

class Round1Round2StateDiff_SBox(AESLeakageHelper):
    name = 'HD: AES Round1/Round2 State diff for SBox'
    def leakage(self, pt, ct, key, bnum):
        state = [pt[i] ^ key[i] for i in range(0, 16)]
        state = self.subbytes(state)
        state1 = state[:]
        state = self.shiftrows(state)
        state = self.mixcolumns(state)

        key2 = self.key_schedule_rounds(key, 0, 1)
        state = [state[i] ^ key2[i] for i in range(0, 16)]
        state = subbytes(state)
        return state[bnum] ^ state1[bnum]

#List of all classes you can use
enc_list = [SBox_output, PtKey_XOR, SBoxInputSuccessive, SBoxInOutDiff, LastroundStateDiff, LastroundStateDiffAlternate, SBoxOutputSuccessive, ShiftColumns_output, Mixcolumns_output, Round1Round2StateDiff_Text, Round1Round2StateDiff_KeyMix, Round1Round2StateDiff_SBox]
dec_list = [InvSBox_output]

class AES128_8bit(ModelsBase):
    """Leakage model for AES128 attacks.

    Make sure to set the actual leakage model (i.e. SBox_Output)
    """
    _name = 'AES 128'

    hwModels = OrderedDict((mod.name, mod) for mod in (enc_list+dec_list) )

    hw_models = OrderedDict((mod.__name__, mod) for mod in (enc_list+dec_list))

    def __init__(self, model=SBox_output, bitmask=0xFF):
        ModelsBase.__init__(self, 16, 256, model=model)
        self.numRoundKeys = 10
        self._mask = bitmask

    def _updateHwModel(self):
        """" Re-implement this to update leakage model """
        self.modelobj = None

        #Check if they passed an object...
        if isinstance(self.model, AESLeakageHelper):
            self.modelobj = self.model

        #Check if they passed a class...
        elif inspect.isclass(self.model) and issubclass(self.model, AESLeakageHelper):
            self.modelobj = self.model()

        #Otherwise it's probably one of these older keys (kept for backwards-compatability)
        else:
            for mod in self.hwModels:
                if (mod.c_model_enum_value == self.model) or (mod.name == self.model):
                    self.modelobj = mod()
                    break

        if self.modelobj is None:
            raise ValueError("Invalid model: %s" % str(self.model))

    def processKnownKey(self, inpkey):
        if hasattr(self.modelobj, 'processKnownKey'):
            return self.modelobj.processKnownKey(inpkey)
        return inpkey

    def leakage(self, pt, ct, guess, bnum, state):
        """ Leakage as set by model

        Args:
            pt (list): Plaintext/textin
            ct (list): Ciphertext/textout
            guess (list): Key guess
            bnum (list): Subkey Byte Number
            state (list): The state of the key finding

        Returns:
            A hamming weight (int)
        """
        try:
            #Make a copy so we don't screw with anything...
            key = list(state['knownkey'])
        except:
            #We don't log due to time-sensitive nature... but if state doesn't have "knownkey" will result in
            #unknown knownkey which causes some attacks to fail. Possibly should make this some sort of
            #flag to indicate we want to ignore the problem?
            key = [None]*16

        #Guess can be 'none' if we want to use original key as-is
        if guess is not None:
            key[bnum] = guess

        #Get intermediate value
        intermediate_value = self.modelobj.leakage(pt, ct, key, bnum)

        #For bit-wise attacks, mask off specific bit value
        intermediate_value = self._mask & intermediate_value

        #Return HW of guess
        return self.HW[intermediate_value]

    def key_schedule_rounds(self, inputkey, inputround, desiredround):
        """Changes the round of inputkey from inputround to desiredround

        Args:
            inputkey (list): key that you want to change the round of
            inputround (int): Round that inputkey is currently in
            desiredround (int): Round that you want inputkey to be in

        Returns:
            desired key (list)
        """
        return key_schedule_rounds(inputkey, inputround, desiredround)

    keyScheduleRounds = camel_case_deprecated(key_schedule_rounds)



#!/usr/bin/pythonh
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2014, NewAE Technology Inc
# All rights reserved.
#
# Author: Colin O'Flynn
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================

import numpy as np

from ._base import PreprocessingBase
from chipwhisperer.common.utils.parameter import setupSetParam


class ResyncPeakDetect(PreprocessingBase):
    """
    Resyncronize based on peak value.
    """
    _name = "Resync: Peak Detect"
    _description = "Line up traces so peak (either max positive or max negative) within" \
    " some given range of points all aligns. For each trace the following must hold or the trace is rejected:\n" \
    "   (1-valid limit) < (peak value from candidate trace) / (peak value from reference) < (1+valid limit)\n" \
    "If 'valid limit' is 0 then this is ignored, and all traces are kept."

    def __init__(self, traceSource=None, name=None):
        PreprocessingBase.__init__(self, traceSource, name=name)
        self._rtrace = 0
        self._ccStart = 0
        self._ccEnd = 0
        self._limit = 0
        self._type = "max"

        self.params.addChildren([
            {'name':'Ref Trace', 'key':'reftrace', 'type':'int', 'get':self._getRefTrace, 'set':self._setRefTrace},
            {'name':'Peak Type', 'key':'peaktype', 'type':'list', 'values':['max', 'min'], 'get':self._getType, 'set':self._setType},
            {'name':'Point Range', 'key':'ptrange', 'type':'rangegraph',  'get':self._getWindow, 'set':self._setWindow},
            {'name':'Valid Limit', 'key':'vlimit', 'type':'float', 'step':0.1, 'limits':(0, 10), 'set':self._setLimit, 'get':self._getLimit},
        ])
        self._calculateRef()

    @setupSetParam("Ref Trace")
    def _setRefTrace(self, num):
        self._rtrace = num
        self._calculateRef()

    def _getRefTrace(self):
        return self._rtrace

    @property
    def ref_trace(self):
        """The trace being used as a reference.

        Setter raises TypeError unless value is an integer."""
        return self._getRefTrace()

    @ref_trace.setter
    def ref_trace(self, num):
        if not isinstance(num, int):
            raise TypeError("Expected int; got %s" % type(num), num)
        self._setRefTrace(num)

    @setupSetParam("Peak Type")
    def _setType(self, type):
        self._type = type
        self._calculateRef()

    def _getType(self):
        return self._type

    @property
    def type(self):
        """Which type of peak to look for.

        Valid values are "max" and "min".

        Setter raises ValueError if value isn't recognized.
        """
        return self._getType()

    @type.setter
    def type(self, type):
        valid = ["min", "max"]
        if type not in valid:
            raise ValueError("Unrecognized peak type; expected 'min' or 'max'", type)

    @setupSetParam("Point Range")
    def _setWindow(self, window):
        self._ccStart, self._ccEnd = window
        self._calculateRef()

    def _getWindow(self):
        return (self._ccStart, self._ccEnd)

    @property
    def range(self):
        """The section of the reference trace used for comparison.

        This must be a tuple of (first point, last point).

        Setter raises TypeError if value is not a tuple or if points are not
        integers.
        """
        return self._getWindow()

    @range.setter
    def range(self, win):
        if not isinstance(win, tuple):
            raise TypeError("Expected tuple; got %s" % type(win), win)
        if not isinstance(win[0], int):
            raise TypeError("Expected int; got %s" % type(win[0]), win[0])
        if not isinstance(win[1], int):
            raise TypeError("Expected int; got %s" % type(win[1]), win[1])
        self._setWindow(win)

    @setupSetParam("Valid Limit")
    def _setLimit(self, lim):
        self._limit = lim
        self._calculateRef()

    def _getLimit(self):
        return self._limit

    @property
    def valid_limit(self):
        """The tolerance level on the peak-matching search.

        A lower valid limit means that the peaks have to be closer in value for
        a trace to be accepted.

        A valid limit of 0 indicates that all traces should be kept.

        Setter raises TypeError if value isn't a float.
        """
        return self._getLimit()

    @valid_limit.setter
    def valid_limit(self, lim):
        if not isinstance(lim, float):
            raise TypeError("Expected float; got %s" % type(lim), lim)
        self._setLimit(lim)

    def updateLimits(self):
        if self._traceSource:
            self.findParam('ptrange').setLimits((0, self._traceSource.numPoints()))

    def updateScript(self, _=None):

        pt = self.findParam('ptrange').getValue()

        self.updateLimits()

    def setReference(self, rtraceno=0, peaktype='max', refrange=(0, 0), validlimit=0):
        self._rtrace = rtraceno
        self._limit = validlimit
        self._type = peaktype
        self._ccStart = refrange[0]
        self._ccEnd = refrange[1]
        self.init()

    def getTrace(self, n):
        if self.enabled:
            trace = self._traceSource.getTrace(n)
            if trace is None:
                return None
            if str.lower(self._type) == 'max':
                newmaxloc = np.argmax(trace[self._ccStart:self._ccEnd])
                maxval = max(trace[self._ccStart:self._ccEnd])
            else:
                newmaxloc = np.argmin(trace[self._ccStart:self._ccEnd])
                maxval = min(trace[self._ccStart:self._ccEnd])

            if self._limit:
                if (maxval > self._refmaxsize * (1.0 + self._limit)) | (maxval < self._refmaxsize * (1.0 - self._limit)):
                    return None

            diff = newmaxloc-self._refmaxloc
            if diff < 0:
                trace = np.append(np.zeros(-diff), trace[:diff])
            elif diff > 0:
                trace = np.append(trace[diff:], np.zeros(diff))
            return trace
        else:
            return self._traceSource.getTrace(n)

    def _calculateRef(self):
        try:
            self.calcRefTrace(self._rtrace)
        except ValueError:
            self.findParam('enabled').setValue(False)

    def calcRefTrace(self, tnum):
        # If not enabled stop
        if self.enabled == False:
            return

        reftrace = self._traceSource.getTrace(tnum)[self._ccStart:self._ccEnd]
        if self._type == 'max':
            self._refmaxloc = np.argmax(reftrace)
            self._refmaxsize = max(reftrace)
        else:
            self._refmaxloc = np.argmin(reftrace)
            self._refmaxsize = min(reftrace)



#
# Copyright (c) 2014-2018, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.chipwhisperer.com
#
#    This file is part of ChipWhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#
# ChipWhisperer is a trademark of NewAE Technology Inc., registered in the
# United States of America, the European Union, and other jurisdictions.
#
#==========================================================================


import asyncio
import usb.backend.libusb0 as libusb0
import usb.core
import usb.util
import queue

allow_kill = True


class CWCommands(object):
    CONNECT = 1
    DISCONNECT = 2
    GET_FW_VERSION = 3
    SET_CTRL = 4
    GET_CTRL = 5
    CMD_READMEM = 6
    CMD_WRITEMEM = 7


    def pack_command(self):
        pass


class CWSerialServerThread(object):
    """"This class handles the serial terminal data (or log data etc)"""

class CWCommandServerThread(object):
    """This class handles the command server"""

class CWUSBThread(object):
    """This class handles the USB communication with ChipWhisperer Hardware"""

    def __init__(self):
        self.queue = queue.Queue()

    def connect(self, serial_number=None, vid=None):
        pass



class CWServerThreadManager(object):

    def __init__(self):

        self.item_template = { 'pid':0,
          'vid':0,
          'type':None,
          'sn':None,
          'status':'CLOSED',
          'cmdport':0,
          'serport':0,
          'thread':None
        }

        self.devices = []


    def get_devices(self, scan_for_new=True):
        """Get list of known and new devices"""

        if scan_for_new:
            newdevs = self.get_usb_devices()
            olddevs = self.devices.copy()

            #Delete matching devices ot end up with only MISSING devices
            for i,d in enumerate(olddevs):
                for n in newdevs:
                    if d['sn'] == n.serial_number:
                        del olddevs[i]

            #Delete matching devices to end up with only UNKNOWN devices
            for d in self.devices:
                for i,n in enumerate(newdevs):
                    if d['sn'] == n.serial_number:
                        del newdevs[i]

            print(newdevs)
            print(olddevs)

            #Found any devices? if so insert in table
            for d in newdevs:
                self.devices.append(self.item_template.copy())
                self.devices[-1]['vid'] = d.idVendor
                self.devices[-1]['pid'] = d.idProduct
                self.devices[-1]['sn'] = d.serial_number

            #Do we have missing devices? If so should shut down the server
            for d in olddevs:
                print("Killing thread for sn: {}".format(d['sn']))

        return self.devices

    def get_usb_devices(self):
        """Scan for USB devices"""

        devlist = []

        try:
            # Connect to device (attempt #1)
            dev = list(usb.core.find(find_all=True, idVendor=0x2B3E, backend=libusb0.get_backend()))
        except usb.core.NoBackendError:
            try:
                # An error in the previous one is often caused by Windows 64-bit not detecting the correct library, attempt to force this with paths
                # that often work so user isn't aware
                dev = list(usb.core.find(find_all=True, idVendor=0x2B3E, backend=libusb0.get_backend(
                    find_library=lambda x: r"c:\Windows\System32\libusb0.dll")))
            except usb.core.NoBackendError:
                raise IOError(
                    "Failed to find USB backend. Check libusb drivers installed, check for path issues on library, and check for 32 vs 64-bit issues.")
        # Found something
        if len(dev) > 0:
            devlist.extend(dev)

        return devlist

    def open(self, sn=None):
        pass


    def close(self, sn=None):
        pass


class CWServerTelnetClient(asyncio.Protocol):
    def print_welcome(self):
        """Print Welcome Message on Connection"""
        self.transport.write("Welcome to ChipWhisperer Hardware Server\r\n".encode())
        self.print_prompt()

    def print_prompt(self):
        """Print the user prompt"""
        self.transport.write("\r\n>".encode())

    def print_help(self):
        """Prints help messages"""
        help_msg = """
  Short  Long   Description\r
  ?      help   This screen.\r
  q      quit   Closes connection.\r
  k      kill   Closes ALl connections, kills server.\r
  l      list   Lists connected USB devices.\r
  o      open   Opens a device. If more than one device available must specify serial number.\r
  c      close  Closes a device. If more than one device open must specify serial number.\r
"""
        self.transport.write(help_msg.encode())

    def process_control_chars(self, input_string):
        """"Process control characters (deal with backspace only for now)"""

        # Initial state
        # String is stored as a list because
        # python forbids the modification of
        # a string
        displayed_string = []
        cursor_position = 0

        # Loop on our input (transitions sequence)
        for character in input_string:

            # Alphanumeric transition
            if str.isprintable(character):
                # Add the character to the string
                displayed_string[cursor_position:cursor_position + 1] = character
                # Move the cursor forward
                cursor_position += 1

            # Backward transition
            elif character == "\x08":
                # Move the cursor backward
                cursor_position -= 1
            elif character == "\r":
                pass
            elif character == "\n":
                pass
            else:
                print("{} is not handled by this function".format(repr(character)))

        # We transform our "list" string back to a real string
        return "".join(displayed_string)

    def process_cmd(self, cmd):
        """Process commands"""

        if cmd == "?" or cmd == "help":
            self.print_help()

        elif cmd == "q" or cmd == "quit":
            self.close()

        elif cmd == "k" or cmd == "kill":
            asyncio.get_event_loop().stop()

        elif cmd == "l" or cmd == "list":
            self.list_usb()

        elif cmd == "o" or cmd == "open":
            pass

        elif cmd == "c" or cmd == "close":
            pass

        else:
            self.transport.write("Unknown command: {}".format(cmd).encode())

    def list_usb(self):
        table_string = "PID:VID\t\tTYPE\t\tSERIALNUM\t\t\t\tSTATUS\tCMDPORT\tSERPORT\r\n"

        devlist = cwmanager.get_devices()

        for d in devlist:
            table_string += hex(d['vid']) + ":"
            table_string += hex(d['pid']) + "\t"
            table_string += "?\t" + "\t"
            table_string += d['sn'] + "\t"

            table_string += "\r\n"

        self.transport.write(table_string.encode())

    def close(self):
        """Close connection, print to screen"""
        print('Closed connection to {}'.format(self.transport.get_extra_info('peername')))
        self.transport.close()

    def connection_made(self, transport):
        """"Called on connection setup."""
        peername = transport.get_extra_info('peername')
        print('Connection from {}'.format(peername))
        self.transport = transport
        self.message_buffer = ""

        self.print_welcome()

    def data_received(self, data):
        """Called when data is received"""
        message = data.decode()

        self.message_buffer += message

        if message[0] == '\x03':
            self.close()

        if "\r" in message or "\n" in message:
            cmd = self.process_control_chars(self.message_buffer)
            cmd = cmd.split()[0]
            self.message_buffer = ""

            self.process_cmd(cmd)
            self.print_prompt()


cwmanager = CWServerThreadManager()

loop = asyncio.get_event_loop()
# Each client connection will create a new protocol instance
coro = loop.create_server(CWServerTelnetClient, '127.0.0.1', 1173)
server = loop.run_until_complete(coro)

# Serve requests until Ctrl+C is pressed
print('Serving on {}'.format(server.sockets[0].getsockname()))
loop.run_forever()

# Close the server
server.close()
loop.run_until_complete(server.wait_closed())
loop.close()


#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2013-2016, NewAE Technology Inc
# All rights reserved.
#
# Find this and more at newae.com - this file is part of the chipwhisperer
# project, http://www.assembla.com/spaces/chipwhisperer
#
#    This file is part of chipwhisperer.
#
#    chipwhisperer is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    chipwhisperer is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with chipwhisperer.  If not, see <http://www.gnu.org/licenses/>.
#=================================================
import logging

from ._base import ReaderTemplate


class ReaderChipWhispererLiteSCard(ReaderTemplate):
    _name = "CW1173/1180-SCARD"
    REQ_DATA = 0x1C
    REQ_CFG = 0x1D
    REQ_AUX = 0x1E
    REQ_CFG_ATR = 0x01
    REQ_CFG_PROTOCOL = 0x02
    REQ_CFG_TXRX = 0x05

    def __init__(self):
        ReaderTemplate.__init__(self)

        self.protocol = 0
        self.getParams().addChildren([
            {'name':'Get ATR (Reset Card)', 'type':'action', 'action':self.reset},
            {'name':'ATR', 'key':'atr', 'type':'str', 'value':""},
        ])

    def sendAPDU(self, cla, ins, p1, p2, txdata=None, rxdatalen=0):
        """Send APDU to SmartCard, get Response"""

        data = [cla,ins, p1, p2]

        if txdata is not None:
            txdatalen = len(txdata)
            data.append(txdatalen)
        else:
            txdatalen = 0

        if (txdata is None) & (rxdatalen == 0):
            data.append(0)

        #Append payload
        if txdata is not None:
            for b in txdata: data.append(b)

        if rxdatalen != 0:
            data.append(rxdatalen)

        if self.protocol:
            #T-1 Mode needs header added
            data.insert(0, len(data))
            data.insert(0, 0)
            data.insert(0, 0)

        #Load APDU into buffer & send
        #TODO: Break APDU into multiple transmissions for larger blocks
        offset = 0
        self.usbcon.sendCtrl(self.REQ_DATA, value=offset, data=data)
        self.usbcon.sendCtrl(self.REQ_CFG, self.REQ_CFG_TXRX)

        #Read response
        if self.protocol:
            contRead = True
            offset = 0
            resplen = 64
            response = []
            while contRead:
                data = self.usbcon.readCtrl(self.REQ_DATA, value = ((offset << 8) | resplen), dlen=resplen)
                response.extend(data)

                if len(data) == resplen:
                    offset += resplen
                else:
                    contRead = False

            #Simple reader passes everything back to us
            logging.info(response)

            if ((response[1] & 0x7F) == 0x81) or((response[1] & 0x7F) == 0x82):
                   raise IOError("Error on response:, T-1 PCB = %02x"%response[1])

            #Get length
            resplen = response[2]
            #Cut off first three bytes & CRC bytes
            response = response[3:(3+resplen)]
            logging.info(response)
            #Get status
            status = (response[0] << 8) | response[1]

            #Return status seperately from "payload" if any
            if len(response) > 2:
                return (status, response[2:])
            else:
                return status
        else:
            #TODO: break into multiple as well for larger blocks?
            offset = 0
            resplen = rxdatalen+2
            response = self.usbcon.readCtrl(self.REQ_DATA, value = ((offset << 8) | resplen), dlen=resplen)
            sw1 = response[0]
            sw2 = response[1]

            status = (sw1 << 8) | sw2;

            if rxdatalen > 0:
                response = response[2:]
                return (status, response)

            return status

    def con(self, scope=None):
        """Connect to reader. oa parameter is OpenADC/ChipWhisperer hardware, only used to integrated readers"""
        self.usbcon = scope.qtadc.ser
        self.reset()

    def flush(self):
        """Discard all input buffers"""
        pass

    def reset(self, _=None):
        """Reset card & save the ATR"""

        self.usbcon.sendCtrl(self.REQ_CFG, self.REQ_CFG_ATR)
        self.atr = self.usbcon.readCtrl(self.REQ_CFG, self.REQ_CFG_ATR, 55)
        stratr = " ".join(["%02x"%t for t in self.atr])
        logging.info('ATR: %s' % stratr)
        self.findParam('atr').setValue(stratr)

        self.protocol = self.usbcon.readCtrl(self.REQ_CFG, self.REQ_CFG_PROTOCOL, 1)[0]

        logging.info('SmartCard Protocol = T-%d' % self.protocol)

    def getATR(self):
        """Get the ATR from the SmartCard. Reads a saved value, user reset() to actually reset card."""
        return self.atr
